import * as React from 'react';
import * as ol from 'ol';
import * as layer from 'ol/layer';
import * as source from 'ol/source';
import * as geom from 'ol/geom';
import * as olStyle from 'ol/style';
import * as olInteraction from 'ol/interaction';
import Feature from 'ol/Feature';
import type { MapOptions } from 'ol/PluggableMap';
import UseCoordinateType from './mapTools/UseCoordinate/index.d';
import UseFormatAreaType from './mapTools/UseFormatArea/index.d';
import Attribution from 'ol/control/Attribution';
import FullScreen from 'ol/control/FullScreen';
import MousePosition from 'ol/control/MousePosition';
import OverviewMap from 'ol/control/OverviewMap';
import Rotate from 'ol/control/Rotate';
import ScaleLine from 'ol/control/ScaleLine';
import Zoom from 'ol/control/Zoom';
import ZoomSlider from 'ol/control/ZoomSlider';
import ZoomToExtent from 'ol/control/ZoomToExtent';
import XYZSource from 'ol/source/XYZ';
import WMTSSource from 'ol/source/WMTS';
import TileArcGISRestSource from 'ol/source/TileArcGISRest';
import Overlay from 'ol/Overlay';
import RenderFeature from 'ol/render/Feature';
import Geometry from 'ol/geom/Geometry';
import MapBrowserEvent from 'ol/MapBrowserEvent';
import ImageStyle from 'ol/style/Image';
import TileSource from 'ol/source/Tile';
/** 地图props类型  */
export declare type OlMyMapProps = {
    mapId: string;
    center?: number[];
    zoom?: number;
    minZoom?: number;
    maxZoom?: number;
    xScale?: number;
    yScale?: number;
    defaultLayersVisible?: boolean;
    otherMapOpts?: MapOptions;
    children?: any;
} & Record<string, any>;
/** refProps */
export declare type OlMyMapRef = {
    /**实例化ol.map对象ref */
    BaseMapRef: {
        mapRef: React.MutableRefObject<ol.Map | undefined>;
    };
    /** 地图所有类型图层(除交互事件图层外)统一操作 */
    UseAllMapLayer: {
        /** 图层显示与隐藏 */
        setLayerVisible: (key: string | string[], visible: boolean) => boolean;
        /** 图层显隐切换 */
        setLayerToggetVisible: (key: string | string[]) => boolean;
        /** 移除图层 */
        removeLayer: (key: string | string[]) => boolean;
    };
    /**地图控制器配置 */
    UseControls: {
        /**
         * 添加控制按钮
         * @param control
         */
        add: (control: Keys<TOOLSTYPE> | Keys<TOOLSTYPE>[]) => void;
        /**
         * 删除控制按钮
         * @param control
         */
        remove: (control: Keys<TOOLSTYPE> | Keys<TOOLSTYPE>[]) => void;
        /**
         * 清空
         */
        clear: () => void;
    };
    /**样式设置 */
    UseStyle: {
        createClusterStyle: (feature: RenderFeature | Feature<geom.Geometry>, styleOpts: Omit<OLMAP.ClusterFeatureType.AddClusterOpts, 'groupId' | 'iconList' | 'distance' | 'styleFn'>) => olStyle.Style;
        createAreaStyle: (feature: {
            text: string;
        } & Pick<OLMAP.AreaFeatureType.AddAreaOpts, "textVisible" | "textParam" | "textOffset" | "textAlign" | "textBaseline" | "font" | "textColor" | "textBackgroundColor" | "textPadding" | "zIndex" | "styleFn" | "isLock" | "defaultVisible" | "backgroundColor" | "borderSize" | "borderColor">) => olStyle.Style;
        createIconStyle: (styleOpts: {
            text: string;
        } & Omit<OLMAP.IconFeatureType.AddIconOpts, 'groupId' | 'iconList'>) => olStyle.Style;
        createLineStyle: (styleOpts: Omit<OLMAP.TrajectoryType.AddTrajectoryOpts, 'id' | 'list'>) => olStyle.Style;
    };
    /**区域处理工具 */
    UseFormatArea: UseFormatAreaType;
    /**点位处理工具 */
    UseCoordinate: UseCoordinateType;
    /**视图处理工具 */
    UseView: {
        setCenter: (coordinate: [number, number]) => boolean;
        setZoom: (zoom: number) => boolean;
        /**
         * 通过wkt/geojson 自适应可视区域
         * @param str wkt/geojson 字符串
         * @param padding [top,right,bottom,left]
         */
        setFit: (str: string, padding: [number, number, number, number]) => boolean;
        getCenter: () => number[] | undefined;
        getZoom: () => number | undefined;
    };
    /**地图底图加载 */
    UseTileLayer: {
        /**地图底图图层ref---- new Map() */
        mapBaseLayerRef: React.MutableRefObject<Map<string, OLMAP.TileType<OLMAP.SourceAllType>>>;
        /**设置天地图矢量底图 */
        setTDTLayer: (options: OLMAP.LayerType.TileLayerOpts & OLMAP.LayerType.TiandituOpts) => boolean;
        /**设置天地图路网 */
        setTDTLWLayer: (options: OLMAP.LayerType.TileLayerOpts & Omit<OLMAP.LayerType.TiandituOpts, 'type'>) => boolean;
        /**设置百度地图 */
        setBaiduLayer: (options: OLMAP.LayerType.TileLayerOpts) => boolean;
        /**设置高德地图 */
        setGaoDeLayer: (options: OLMAP.LayerType.TileLayerOpts & OLMAP.LayerType.GaodeLayerOpts) => boolean;
        /**
         * 添加影像切片图层
         * @param options TileLayer配置项
         */
        setYXLayer: (options: OLMAP.LayerType.TileLayerOpts & {
            access_token?: string;
            curl?: string;
            sku?: string;
        }) => boolean;
        /**设置自定义地图 */
        setUpYouLayer: (options: OLMAP.LayerType.TileUpYouLayerOpts) => boolean;
        /**设置arcgisrect */
        setTileArcGISRestLayer: (options: OLMAP.LayerType.TileArcGISRestLayerOpts) => boolean;
        /**设置wmts图层 */
        setWmtsLayer: (options: OLMAP.LayerType.TileWmtsLayerOpts) => boolean;
        /**移除图层 */
        removeLayer: (key: string | string[]) => boolean;
        /**设置图层visible */
        setVisible: (classname: string | string[], visible: boolean) => boolean;
        setToggetVisible: (classname: string | string[]) => boolean;
        isVisible: (classname: string | string[]) => boolean;
        /**隐藏所有的底图图层 */
        hideAll: () => boolean;
        /**显示所有的底图图层 */
        showAll: () => boolean;
        /**清楚图层 */
        clear: () => boolean;
    };
    /**边界图层 */
    UseAreaFeatures: {
        /**地图边界图层ref---- new Map() */
        areaMapLayerRef: React.MutableRefObject<Map<string, OLMAP.areaMapLayerType>>;
        add: (opts: OLMAP.AreaFeatureType.AddAreaOpts) => false | Feature<any>[];
        /**移除图层 */
        remove: (groupId: string | string[]) => boolean;
        setVisible: (groupId: string | string[], visible: boolean) => boolean;
        isVisible: (groupId: string | string[]) => boolean;
        setToggetVisible: (groupId: string | string[]) => boolean;
        clear: () => boolean;
    };
    /** 图标图层 */
    UseIconFeatures: {
        /**地图点位图层ref---- new Map() */
        iconFeatureLayerRef: React.MutableRefObject<Map<string, OLMAP.iconFeatureLayerType>>;
        add: (opts: OLMAP.IconFeatureType.AddIconOpts) => false | Feature<geom.Point>[];
        remove: (groupId: string | string[]) => boolean;
        setVisible: (groupId: string | string[], visible: boolean) => boolean;
        setToggetVisible: (groupId: string | string[]) => boolean;
        isVisible: (groupId: string | string[]) => boolean;
        clear: () => boolean;
        /**
         * 修改位置
         * @param groupId 组id
         * @param find 类似 list.find
         * @param coordinate 经纬度
         */
        setPosition: (groupId: string, find: (data: {
            userData: Icon;
        }) => boolean, coordinate: number[]) => any;
        focus: (groupId: string | string[], padding?: [number, number, number, number]) => boolean | undefined;
        check: (groupId: string) => boolean;
    };
    /**聚合点位加载 */
    UseClusterFeatures: {
        /**地图聚合点位图层ref---- new Map() */
        clusterFeatureLayerRef: React.MutableRefObject<Map<string, OLMAP.clusterFeatureLayerType>>;
        add: (opts: OLMAP.ClusterFeatureType.AddClusterOpts) => false | Feature<geom.Point>[];
        remove: (groupId: string | string[]) => boolean;
        setVisible: (groupId: string | string[], visible: boolean) => boolean;
        setToggetVisible: (groupId: string | string[]) => boolean;
        isVisible: (groupId: string | string[]) => boolean;
        clear: () => boolean;
        focus: (groupId: string | string[], padding?: [number, number, number, number]) => boolean | undefined;
        check: (groupId: string) => boolean;
    };
    /**覆盖层加载 */
    UseOverlayFeatures: {
        positionMap: React.MutableRefObject<Map<string, number[] | undefined>>;
        add: (opts: OLMAP.OverlayFeaturesType.AddOverlayerOpts) => false | OLMAP.OverlayLayerType;
        remove: (id: string) => boolean;
        setVisible: (id: string, visible: boolean) => boolean;
        setToggetVisible: (id: string) => boolean;
        isVisible: (id: string) => boolean;
        clear: () => boolean;
        setPosition: (id: string, coordinate: [number, number] | number[]) => boolean;
    };
    /**覆盖层定时轮播加载 */
    UseOverlayChange: {
        add: (opts: OLMAP.OverlayChangeType.OverlayChangeOpts) => boolean | undefined;
        remove: (id: string) => boolean;
        setVisible: (id: string, visible: boolean) => boolean;
        setToggetVisible: (id: string) => boolean;
        isVisible: (id: string) => boolean;
        clear: () => boolean;
        stop: (id: string) => boolean;
        start: (id: string) => boolean;
    };
    /**轨迹加载 */
    UseTrajectory: {
        trajectoryLayerMap: React.MutableRefObject<Map<string, OLMAP.trajectoryLayerType>>;
        add: (opts: OLMAP.TrajectoryType.AddTrajectoryOpts) => boolean;
        remove: (id: string) => boolean;
        setVisible: (id: string, visible: boolean) => boolean;
        setToggetVisible: (id: string) => boolean;
        isVisible: (id: string) => boolean;
        clear: () => boolean;
        restart: (id: string) => boolean;
    };
    /**事件绑定 */
    UseEvent: {
        listenerRef: React.MutableRefObject<Record<OLMAP.UseEventType.EventType, Record<string, (e: OLMAP.UseEventType.MapBrowserEventType) => void> | undefined>>;
        /**
         * 添加监听事件
         * @param key 索引
         * @param type 事件类型
         * @param listener 监听回调函数
         */
        addEventListener: (key: string, type: OLMAP.UseEventType.EventType, listener: (e: OLMAP.UseEventType.MapEvent) => void) => void;
        /**
         * 移除监听事件 （key若不传，则清除全部type）
         * @param type 事件类型
         * @param key 索引
         */
        removeEventListener: (type: OLMAP.UseEventType.EventType, key?: string | string[] | undefined) => void;
        /**
         * 清空所有事件，组件卸载时使用，防止事件清空遗漏。
         */
        clearEventListener: () => void;
    };
    /**地图交互 */
    UseInteraction: {
        /**地图交互，点击样式select */
        addSelect: (opts: OLMAP.UseInteractionType.AddInteractionSelectOpts) => boolean | olInteraction.Select;
        /**交互-画线，点，面 */
        addDraw: (opts: OLMAP.UseInteractionType.AddInteractionDrawOpts) => boolean | olInteraction.Draw;
        remove: (interactionKey: string | string[]) => boolean;
        clear: () => boolean;
    };
};
export declare namespace OLMAP {
    namespace LayerType {
        type TiandituOpts = {
            /**
             * 天地图服务器二级域名
             * @default t0
             */
            domain?: 't0' | 't1' | 't2' | 't3' | 't4' | 't5' | 't6' | 't7' | 't{0-6}';
            /**
             * 天地图服务密钥
             * @description 测试密钥，一天一万次调用，请根据项目申请！！！！！ https://console.tianditu.gov.cn/api/key
             * @default 8abf1ede052e23d2322a54905de565a1
             */
            tk?: string;
            greyMask?: boolean;
            type: 'vec_w' | 'cva_w' | 'img_w' | 'cia_w' | 'ter_w' | 'cta_w' | 'ibo_w' | 'vec_c' | 'cva_c' | 'img_c' | 'cia_c' | 'ter_c' | 'cta_c' | 'ibo_c';
        };
        type TileLayerOpts = {
            /** Opacity (0, 1). */
            opacity?: number;
            /** Visibility */
            visible?: boolean;
            /**
             * 层级
             * @default -1
             */
            zIndex?: number;
            /** The minimum resolution (inclusive) at which this layer will be visible. */
            minResolution?: number;
            maxResolution?: number;
            minZoom?: number;
            maxZoom?: number;
            extent?: any;
            properties?: any;
            source?: XYZSource | WMTSSource;
        } & Record<string, any>;
        type BaiduLayerOpts = {
            /** Opacity (0, 1). */
            opacity?: number | undefined;
            /** Visibility */
            visible?: boolean | undefined;
            /**
             * 层级
             * @default -1
             */
            zIndex?: number | undefined;
            /** The minimum resolution (inclusive) at which this layer will be visible. */
            minResolution?: number | undefined;
            maxResolution?: number | undefined;
            minZoom?: number | undefined;
            maxZoom?: number | undefined;
            extent?: any;
            properties?: any;
            source?: XYZSource | undefined;
        } & Record<string, any> & {
            tileUrlFunction?: ((e: number[]) => string) | undefined;
            customid?: "normal" | "light" | "dark" | "redalert" | "googlelite" | "grassgreen" | "midnight" | "pink" | "darkgreen" | "bluish" | "grayscale" | "hardedge" | undefined;
        };
        type GaodeLayerOpts = {
            type?: '道路带注记' | '道路不带注记' | '矢量带标注' | '影像不带注记' | '矢量地图带注记' | '矢量地图不带注记';
        } & Record<string, any>;
        type TileUpYouLayerOpts = {
            /** Opacity (0, 1). */
            opacity?: number | undefined;
            /** Visibility */
            visible?: boolean | undefined;
            /**
             * 层级
             * @default -1
             */
            zIndex?: number | undefined;
            /** The minimum resolution (inclusive) at which this layer will be visible. */
            minResolution?: number | undefined;
            maxResolution?: number | undefined;
            minZoom?: number | undefined;
            maxZoom?: number | undefined;
            extent?: any;
            properties?: any;
            source?: XYZSource | undefined;
        } & Record<string, any> & {
            key: string;
        };
        type TileArcGISRestLayerOpts = {
            /** Opacity (0, 1). */
            opacity?: number | undefined;
            /** Visibility */
            visible?: boolean | undefined;
            /**
             * 层级
             * @default -1
             */
            zIndex?: number | undefined;
            /** The minimum resolution (inclusive) at which this layer will be visible. */
            minResolution?: number | undefined;
            maxResolution?: number | undefined;
            minZoom?: number | undefined;
            maxZoom?: number | undefined;
            extent?: any;
            properties?: any;
            source?: TileArcGISRestSource | undefined;
        } & Record<string, any> & {
            url: string;
            key: string;
            arcGisRestopts?: OLMAP.SourceType.ArcGisRest;
        };
        type TileWmtsLayerOpts = {
            /** Opacity (0, 1). */
            opacity?: number | undefined;
            /** Visibility */
            visible?: boolean | undefined;
            /**
             * 层级
             * @default -1
             */
            zIndex?: number | undefined;
            /** The minimum resolution (inclusive) at which this layer will be visible. */
            minResolution?: number | undefined;
            maxResolution?: number | undefined;
            minZoom?: number | undefined;
            maxZoom?: number | undefined;
            extent?: any;
            properties?: any;
            source?: TileArcGISRestSource | undefined;
        } & Record<string, any> & {
            url: string;
            key: string;
            wmtsSourceOpts?: OLMAP.SourceType.WMTS;
        };
    }
    namespace AreaFeatureType {
        type AddAreaOpts = {
            /** 组Id */
            groupId: string;
            /**
             * 背景颜色
             * @default rgba(137,119,251,0.5)
             */
            backgroundColor?: string;
            /**
             * 边框大小
             * @default 2
             */
            borderSize?: number;
            /**
             * 边框颜色
             * @default #fff
             */
            borderColor?: string;
            /**
             * 文本显示字段
             * @default name
             */
            textParam?: string;
            /** 自定义样式方法 */
            styleFn?: (icon: Area) => olStyle.Style;
            /** 区域列表 */
            areaList: Area[];
            /**
             * 数据锁
             * @description true为锁定，只允许添加一次，false可覆盖上次
             * @default true
             */
            isLock?: boolean;
            /**
             * 默认可见性
             * @default true
             */
            defaultVisible?: boolean;
            /**
             * 层级
             * @default 10
             */
            zIndex?: number;
        } & TextOpts;
    }
    namespace IconFeatureType {
        type AddIconOpts = {
            /** 组Id */
            groupId: string;
            /** 图标地址（iconList的src优先级高于外部） */
            iconSrc: string;
            /** 自定义样式方法 */
            styleFn?: (icon: Icon) => olStyle.Style;
            /** 图标列表 */
            iconList: Icon[];
            /**
             * 数据锁
             * @description true为锁定，只允许添加一次，false可覆盖上次
             * @default true
             */
            isLock?: boolean;
            /**
             * 默认可见性
             * @default true
             */
            defaultVisible?: boolean;
            /**
             * 层级
             * @default 20
             */
            zIndex?: number;
        } & TextOpts & IconOpts;
    }
    namespace ClusterFeatureType {
        type AddClusterOpts = {
            /** 只有一个的时候展示的图标 */
            onlyOneSrc: string;
            /** 聚合组展示的图标 */
            groupSrc: string;
            /**
             * 聚合范围 m
             * @default 200
             */
            distance?: number;
            /** 自定义样式方法 */
            styleFn?: (feature: RenderFeature | Feature<geom.Geometry>, styleOpts: Omit<OLMAP.ClusterFeatureType.AddClusterOpts, 'groupId' | 'iconList' | 'distance' | 'styleFn'>) => olStyle.Style;
            /** 图标列表 */
            iconList: Icon[];
            /**
             * 数据锁
             * @description true为锁定，只允许添加一次，false可覆盖上次
             * @default true
             */
            isLock?: boolean;
            /**
             * 是否将文本显示聚合图标数量
             * @default false
             */
            isShowPointerNum?: boolean;
            /**
             * 层级
             * @default 20
             */
            zIndex?: number;
        } & Omit<OLMAP.IconFeatureType.AddIconOpts, 'iconList' | 'iconSrc' | 'styleFn'>;
    }
    namespace OverlayFeaturesType {
        /** 新增浮动层配置 */
        type AddOverlayerOpts = {
            /** 索引 */
            id: string;
            /**
             * html元素
             * @description html元素可以提前配置display:none
             */
            element: HTMLElement | HTMLDivElement;
            /**
             * 设置Overlay显示的坐标点
             * @description undefined隐藏
             */
            position?: [number, number] | number[];
            /**UseInteractionType
             * 设置Overlay在position处的位置
             * @default bottom-center
             */
            positioning?: 'bottom-left' | 'bottom-center' | 'bottom-right' | 'center-left' | 'center-center' | 'center-right' | 'top-left' | 'top-center' | 'top-right';
            /**
             * 阻止事件冒泡
             * @default true
             */
            stopEvent?: boolean;
            /**
             * 偏移量
             * @default [0,0]
             */
            offset?: [number, number];
        };
    }
    namespace OverlayChangeType {
        type OverlayChangeOpts<T = {}> = {
            /**
             * 时间间隔（s）
             * @default 4
             */
            time?: number;
            pointerList: (T & Pointer)[];
            getChangeDataFn?: (pointer: T & Pointer) => void;
        } & Omit<OLMAP.OverlayFeaturesType.AddOverlayerOpts, 'position'>;
    }
    namespace TrajectoryType {
        type AddTrajectoryOpts = {
            /** 索引 */
            id: string;
            /**
             * 边框大小
             * @default 4
             */
            borderSize?: number;
            /**
             * 边框颜色
             * @default #f00
             */
            borderColor?: string;
            /**
             * 插值
             * @description 插值越大越平滑
             * @default 40
             */
            interpolation?: number;
            /** 自定义样式方法 */
            styleFn?: (feature: RenderFeature | Feature<geom.Geometry>) => olStyle.Style;
            /** 实时获取经纬度 */
            getCoordinate?: (coordinate: [number, number]) => void;
            /** 经纬度列表 */
            list: TrajData[];
            /**
             * 数据锁
             * @description true为锁定，只允许添加一次，false可覆盖上次
             * @default true
             */
            isLock?: boolean;
            /**
             * 层级
             * @default 30
             */
            zIndex?: number;
        };
    }
    namespace UseEventType {
        type EventType = 'change' | 'click' | 'singleclick' | 'dblclick' | 'moveend' | 'movestart' | 'pointerdrag' | 'pointermove';
        type MapEvent = {
            iconFeatures?: OLMAP.UseEventType.FeaturesType[];
            areaFeatures?: OLMAP.UseEventType.FeaturesType[];
            clusterFeatures?: OLMAP.UseEventType.FeaturesType[];
            clickFeatures?: OLMAP.UseEventType.FeaturesType[];
            clickType: 'icon' | 'area' | 'cluster';
            features?: OLMAP.UseEventType.FeaturesType[];
            mapCenter?: number[];
            clickCoordinate?: number;
        } & MapBrowserEvent<any>;
        type MapBrowserEventType = MapBrowserEvent<any>;
        type FeaturesType = (RenderFeature | Feature<Geometry>) & {
            userData?: any;
        };
        /**
         * altKeyOnly 如果只按下 alt 键
         * altShiftKeysOnly 如果仅按下 alt 键和 shift 键，则返回
         * always 始终
         * click 单击
         * doubleClick 双击
         * focus 聚焦
         * mouse Onlyly 鼠标事件
         * never 从不
         * noModifierKeys 如果未按任何修饰键（alt 键、shift 键或平台修饰键）
         * penOnly 事件源自数字笔
         * platformModifierKeyOnly 如果只按下平台修饰键（Mac 上的元键，否则为 ctrl 键）
         * pointerMove 浏览器事件
         * primaryAction 如果事件源自与曲面接触的主指针或按下鼠标左键
         * shiftKeyOnly 如果只按下 shift 键
         * singleClick 地图单击事件
         * targetNotitable 如果目标元素不可编辑
         * touchOnly 事件源自可触摸设备
         */
        type ConditionType = 'altKeyOnly' | 'altShiftKeysOnly' | 'always' | 'click' | 'doubleClick' | 'focus' | 'mouse' | 'never' | 'noModifierKeys' | 'penOnly' | 'platformModifierKeyOnly' | 'pointerMove' | 'primaryAction' | 'shiftKeyOnly' | 'singleClick' | 'targetNotitable' | 'touchOnly' | string;
    }
    namespace UseInteractionType {
        type AddInteractionSelectOpts = {
            interactionKey: string;
            layerKeys: string[];
            eventType: "change" | "error" | "select" | "propertychange" | "change:active";
            eventCallback?: (e: any) => void;
            styleFn?: (feature: OLMAP.UseEventType.FeaturesType) => olStyle.Style;
        } & Record<string, any>;
        type AddInteractionDrawOpts = {
            interactionKey: string;
            style?: olStyle.Style;
            drawType?: 'Polygon' | 'LineString' | 'MultiPolygon' | 'MultiLineString' | 'LinearRing' | 'Point' | 'MultiPoint' | 'GeometryCollection' | 'Circle';
            clickTolerance?: number;
            features?: any;
            snapTolerance?: number;
            maxPoints?: number;
            minPoints?: number;
            geometryFunction?: (coordinates: any, geometry: OLMAP.GeometryType) => OLMAP.GeometryType;
            geometryName?: string;
            condition?: OLMAP.UseEventType.ConditionType;
            eventList?: ({
                eventType?: "change" | "error" | "propertychange" | "change:active" | "drawstart" | "drawend" | "drawabort";
                eventCallback?: (e: any) => any;
            })[];
            snapOpts?: {
                isAdd?: boolean;
                features?: any;
                edge?: boolean;
                vertex?: boolean;
                pixelTolerance?: number;
                source?: any;
                eventList?: ({
                    eventType?: "change" | "error" | "propertychange" | "change:active";
                    eventCallback?: (e: any) => any;
                })[];
            } & Record<string, any>;
            modifyOpts?: {
                isAdd?: boolean;
                condition?: any;
                deleteCondition?: any;
                insertVertexCondition?: any;
                pixelTolerance?: number;
                style?: olStyle.Style;
                source?: any;
                hitDetection?: boolean;
                features?: any;
                wrapX?: boolean;
                snapToPointer?: boolean;
                eventList?: ({
                    eventType?: "change" | "error" | "propertychange" | "change:active" | "modifystart" | "modifyend";
                    eventCallback?: (e: any) => any;
                })[];
            } & Record<string, any>;
            source?: any;
        } & Record<string, any>;
    }
    type TileType<T extends TileSource> = layer.Tile<T>;
    type SourceAllType = source.WMTS | source.XYZ | source.TileArcGISRest;
    type areaMapLayerType = layer.Vector<source.Vector<geom.Geometry | geom.Circle | geom.GeometryCollection | (geom.Polygon & Record<string, any>)>>;
    type iconFeatureLayerType = layer.Vector<source.Vector<geom.Geometry | geom.Circle | geom.GeometryCollection | geom.Polygon | geom.Point>>;
    type clusterFeatureLayerType = layer.Vector<source.Cluster>;
    type OverlayLayerType = Overlay;
    type trajectoryLayerType = layer.Vector<source.Vector<geom.Geometry | geom.Circle | geom.GeometryCollection | (geom.Polygon & Record<string, any>)>>;
    type GeometryType = geom.Polygon | geom.MultiPolygon | geom.LineString | geom.Point | geom.MultiPoint | geom.GeometryCollection | geom.LinearRing;
    namespace SourceType {
        type ArcGisRest = {
            params?: {
                [key: string]: any;
            };
            hidpi?: boolean;
            crossOrigin?: string | undefined;
            wrapX?: boolean;
        } & Record<string, any> & TileImageBase;
        type WMTS = {
            layer?: string;
            matrixSet?: string;
            format?: string;
            projection?: any;
            tileGrid?: any;
            style?: string;
            wrapX?: boolean;
        } & source.WMTS & TileImageBase;
        type XYZ = {} & source.XYZ & TileImageBase;
        type TileImageBase = {
            attributions?: any;
            attributionsCollapsible?: boolean;
            cacheSize?: number;
            crossOrigin?: string | undefined;
            imageSmoothing?: boolean;
            interpolate?: boolean;
            opaque?: boolean;
            projection?: any;
            reprojectionErrorThreshold?: number;
            state?: any;
            tileClass?: any;
            tileGrid?: any;
            tileLoadFunction?: any;
            tilePixelRatio?: number;
            tileUrlFunction?: any;
            url?: string;
            wrapX?: boolean;
            transition?: number;
            urls?: string[];
            key?: string;
            zDirection?: number;
        };
    }
}
declare type TrajData = {
    lat?: number;
    lng?: number;
} & Record<string, any>;
declare type Pointer = {
    lng?: number;
    lat?: number;
} & Record<string, any>;
/** 区域对象 */
declare type Area = {
    lng?: number;
    lat?: number;
    wkt?: string;
} & Omit<OLMAP.AreaFeatureType.AddAreaOpts, 'groupId' | 'textParam' | 'styleFn' | 'areaList'> & Record<string, any>;
declare type Icon = {
    lng?: number;
    lat?: number;
    iconSrc?: string;
} & Omit<OLMAP.IconFeatureType.AddIconOpts, 'iconSrc' | 'groupId' | 'textParam' | 'styleFn' | 'iconList' | 'defaultVisible'> & Record<string, any>;
export declare type IconOpts = {
    /**
     * imgsource
     */
    image?: ImageStyle;
    /**
     * 偏移量
     * @default [0.5,30]
     */
    iconAnchor?: [number, number];
    /**
     * 原点位置，取值为bottom-left, bottom-right, top-left 或 top-right
     * @default top-right
     */
    iconAnchorOrigin?: 'bottom-left' | 'bottom-right' | 'top-left' | 'top-right';
    /**
     * 透明度
     * @default 1
     */
    iconOpacity?: number;
    /**
     * 缩放
     * @default 0.4
     */
    iconScale?: number;
};
export declare type TextOpts = {
    /**
     * 文字可见性
     * @default true
     */
    textVisible?: boolean;
    /**
     * 文本显示字段
     * @default name
     */
    textParam?: string;
    /**
     * 文字偏移量
     * @default [20,0]
     */
    textOffset?: [number, number];
    /**
     * 取值“ left”，“ right”，“ center”，“ end”或“ start”。
     * @default left
     */
    textAlign?: 'left' | 'right' | 'center' | 'end' | 'start';
    /**
     * 文本的基准线 取值范围: ‘bottom’, ‘top’, ‘middle’, ‘alphabetic’, ‘hanging’, ‘ideographic’.
     * @default middle
     */
    textBaseline?: 'bottom' | 'top' | 'middle' | 'alphabetic' | 'hanging' | 'ideographic';
    /**
     * 字体样式
     * @default '12px PingFang'
     */
    font?: string;
    /**
     * 文本颜色
     * @default #fff
     */
    textColor?: string;
    /**
     * 文本背景颜色
     * @default rgba(0,0,0,.5)
     */
    textBackgroundColor?: string;
    /**
     * 内边距
     * @default [3,8,3,8]
     */
    textPadding?: [number, number, number, number];
};
export declare type Keys<T> = T extends object ? {
    [K in keyof T]-?: K;
}[keyof T] : never;
export declare type TOOLSTYPE = {
    FullScreen: FullScreen;
    Attribution: Attribution;
    MousePosition: MousePosition;
    OverviewMap: OverviewMap;
    Rotate: Rotate;
    ScaleLine: ScaleLine;
    Zoom: Zoom;
    ZoomSlider: ZoomSlider;
    ZoomToExtent: ZoomToExtent;
};
declare const _default: React.MemoExoticComponent<React.ForwardRefExoticComponent<Pick<OlMyMapProps, string> & React.RefAttributes<OlMyMapRef>>>;
export default _default;
