/*
 * @Author: zhenxing xiezhenxing@codvision.com
 * @Date: 2022-05-24 10:17:14
 * @LastEditors: zhenxing xiezhenxing@codvision.com
 * @LastEditTime: 2022-08-22 14:27:54
 * @FilePath: \组件库\src\pages\MapComponent\OlTest\components\OlMap\data.d.ts
 * @Description: 这是默认设置,请设置`customMade`, 打开koroFileHeader查看配置 进行设置: https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 */
import Attribution from 'ol/control/Attribution';
import FullScreen from 'ol/control/FullScreen';
import MousePosition from 'ol/control/MousePosition';
import OverviewMap from 'ol/control/OverviewMap';
import Rotate from 'ol/control/Rotate';
import ScaleLine from 'ol/control/ScaleLine';
import Zoom from 'ol/control/Zoom';
import ZoomSlider from 'ol/control/ZoomSlider';
import ZoomToExtent from 'ol/control/ZoomToExtent';
import XYZSource from 'ol/source/XYZ';
import WMTSSource from 'ol/source/WMTS';
import ol from 'ol';
import TileArcGISRestSource from 'ol/source/TileArcGISRest';
import * as layer from 'ol/layer';
import * as source from 'ol/source';
import * as geom from 'ol/geom';
import * as olStyle from 'ol/style';
import Overlay from 'ol/Overlay';
import RenderFeature from 'ol/render/Feature';
import Feature from 'ol/Feature';
import Geometry from 'ol/geom/Geometry';
import MapBrowserEvent from 'ol/MapBrowserEvent';
import ImageStyle from 'ol/style/Image';
import TileSource from 'ol/source/Tile';
import type { MapOptions } from 'ol/PluggableMap';
import UseCoordinateType from './mapTools/UseCoordinate/index.d';
import UseFormatAreaType from './mapTools/UseFormatArea/index.d';
import * as olInteraction from 'ol/interaction';

/** 地图props类型  */
export type OlMyMapProps = {
  mapId: string;
  center?: number[];
  zoom?: number;
  minZoom?: number;
  maxZoom?: number;
  xScale?: number;
  yScale?: number;
  // 默认图层加载
  defaultLayersVisible?: boolean;
  otherMapOpts?: MapOptions;
  children?: any;
} & Record<string, any>;

/** refProps */
export type OlMyMapRef = {
  /**实例化ol.map对象ref */
  BaseMapRef: {
    mapRef: React.MutableRefObject<ol.Map | undefined>;
  };
  /** 地图所有类型图层(除交互事件图层外)统一操作 */
  UseAllMapLayer: {
    /** 图层显示与隐藏 */
    setLayerVisible: (key: string | string[], visible: boolean) => boolean;
    /** 图层显隐切换 */
    setLayerToggetVisible: (key: string | string[]) => boolean;
    /** 移除图层 */
    removeLayer: (key: string | string[]) => boolean;
  },
  /**地图控制器配置 */
  UseControls: {
    /**
     * 添加控制按钮
     * @param control
     */
    add: (control: Keys<TOOLSTYPE> | Keys<TOOLSTYPE>[]) => void;
    /**
     * 删除控制按钮
     * @param control
     */
    remove: (control: Keys<TOOLSTYPE> | Keys<TOOLSTYPE>[]) => void;
    /**
     * 清空
     */
    clear: () => void;
  },
  /**样式设置 */
  UseStyle: {
    createClusterStyle: (feature: RenderFeature | Feature<geom.Geometry>, styleOpts: Omit<OLMAP.ClusterFeatureType.AddClusterOpts, 'groupId' | 'iconList' | 'distance' | 'styleFn'>) => olStyle.Style,
    createAreaStyle: (feature: {
      text: string;
    } & Pick<OLMAP.AreaFeatureType.AddAreaOpts, "textVisible" | "textParam" | "textOffset" | "textAlign" | "textBaseline" | "font" | "textColor" | "textBackgroundColor" | "textPadding" | "zIndex" | "styleFn" | "isLock" | "defaultVisible" | "backgroundColor" | "borderSize" | "borderColor">) => olStyle.Style,
    createIconStyle: (styleOpts: {
      text: string;
    } & Omit<OLMAP.IconFeatureType.AddIconOpts, 'groupId' | 'iconList'>) => olStyle.Style,
    createLineStyle: (styleOpts: Omit<OLMAP.TrajectoryType.AddTrajectoryOpts, 'id' | 'list'>) => olStyle.Style,
  };
  /**区域处理工具 */
  UseFormatArea: UseFormatAreaType,
  /**点位处理工具 */
  UseCoordinate: UseCoordinateType,
  /**视图处理工具 */
  UseView: {
    setCenter: (coordinate: [number, number]) => boolean;
    setZoom: (zoom: number) => boolean;
    /**
     * 通过wkt/geojson 自适应可视区域
     * @param str wkt/geojson 字符串
     * @param padding [top,right,bottom,left]
     */
    setFit: (str: string, padding: [number, number, number, number]) => boolean;
    getCenter: () => number[] | undefined;
    getZoom: () => number | undefined;
  },
  /**地图底图加载 */
  UseTileLayer: {
    /**地图底图图层ref---- new Map() */
    mapBaseLayerRef: React.MutableRefObject<Map<string, OLMAP.TileType<OLMAP.SourceAllType>>>;
    /**设置天地图矢量底图 */
    setTDTLayer: (options: OLMAP.LayerType.TileLayerOpts & OLMAP.LayerType.TiandituOpts) => boolean,
    /**设置天地图路网 */
    setTDTLWLayer: (options: OLMAP.LayerType.TileLayerOpts & Omit<OLMAP.LayerType.TiandituOpts, 'type'>) => boolean,
    /**设置百度地图 */
    setBaiduLayer: (options: OLMAP.LayerType.TileLayerOpts) => boolean,
    /**设置高德地图 */
    setGaoDeLayer: (options: OLMAP.LayerType.TileLayerOpts & OLMAP.LayerType.GaodeLayerOpts) => boolean,
    /**
     * 添加影像切片图层
     * @param options TileLayer配置项
     */
    setYXLayer: (options: OLMAP.LayerType.TileLayerOpts & { access_token?: string, curl?: string, sku?: string }) => boolean,
    /**设置自定义地图 */
    setUpYouLayer: (options: OLMAP.LayerType.TileUpYouLayerOpts) => boolean,
    /**设置arcgisrect */
    setTileArcGISRestLayer: (options: OLMAP.LayerType.TileArcGISRestLayerOpts) => boolean,
    /**设置wmts图层 */
    setWmtsLayer: (options: OLMAP.LayerType.TileWmtsLayerOpts) => boolean,
    /**移除图层 */
    removeLayer: (key: string | string[]) => boolean,
    /**设置图层visible */
    setVisible: (classname: string | string[], visible: boolean) => boolean
    setToggetVisible: (classname: string | string[]) => boolean,
    isVisible: (classname: string | string[]) => boolean,
    /**隐藏所有的底图图层 */
    hideAll: () => boolean;
    /**显示所有的底图图层 */
    showAll: () => boolean;
    /**清楚图层 */
    clear: () => boolean;
  },
  /**边界图层 */
  UseAreaFeatures: {
    /**地图边界图层ref---- new Map() */
    areaMapLayerRef: React.MutableRefObject<Map<string, OLMAP.areaMapLayerType>>;
    add: (opts: OLMAP.AreaFeatureType.AddAreaOpts) => false | Feature<any>[];
    /**移除图层 */
    remove: (groupId: string | string[]) => boolean,
    setVisible: (groupId: string | string[], visible: boolean) => boolean;
    isVisible: (groupId: string | string[]) => boolean;
    setToggetVisible: (groupId: string | string[]) => boolean;
    clear: () => boolean;
  },
  /** 图标图层 */
  UseIconFeatures: {
    /**地图点位图层ref---- new Map() */
    iconFeatureLayerRef: React.MutableRefObject<Map<string, OLMAP.iconFeatureLayerType>>;
    add: (opts: OLMAP.IconFeatureType.AddIconOpts) => false | Feature<geom.Point>[];
    remove: (groupId: string | string[]) => boolean;
    setVisible: (groupId: string | string[], visible: boolean) => boolean;
    setToggetVisible: (groupId: string | string[]) => boolean;
    isVisible: (groupId: string | string[]) => boolean;
    clear: () => boolean;
    /**
     * 修改位置
     * @param groupId 组id
     * @param find 类似 list.find
     * @param coordinate 经纬度
     */
    setPosition: (groupId: string, find: (data: {
      userData: Icon;
    }) => boolean, coordinate: number[]) => any;
    focus: (groupId: string | string[], padding?: [number, number, number, number]) => boolean | undefined;
    check: (groupId: string) => boolean;
  },
  /**聚合点位加载 */
  UseClusterFeatures: {
    /**地图聚合点位图层ref---- new Map() */
    clusterFeatureLayerRef: React.MutableRefObject<Map<string, OLMAP.clusterFeatureLayerType>>;
    add: (opts: OLMAP.ClusterFeatureType.AddClusterOpts) => false | Feature<geom.Point>[];
    remove: (groupId: string | string[]) => boolean;
    setVisible: (groupId: string | string[], visible: boolean) => boolean;
    setToggetVisible: (groupId: string | string[]) => boolean;
    isVisible: (groupId: string | string[]) => boolean;
    clear: () => boolean;
    focus: (groupId: string | string[], padding?: [number, number, number, number]) => boolean | undefined;
    check: (groupId: string) => boolean;
  },
  /**覆盖层加载 */
  UseOverlayFeatures: {
    positionMap: React.MutableRefObject<Map<string, number[] | undefined>>;
    add: (opts: OLMAP.OverlayFeaturesType.AddOverlayerOpts) => false | OLMAP.OverlayLayerType;
    remove: (id: string) => boolean;
    setVisible: (id: string, visible: boolean) => boolean;
    setToggetVisible: (id: string) => boolean;
    isVisible: (id: string) => boolean;
    clear: () => boolean;
    setPosition: (id: string, coordinate: [number, number] | number[]) => boolean;
  },
  /**覆盖层定时轮播加载 */
  UseOverlayChange: {
    add: (opts: OLMAP.OverlayChangeType.OverlayChangeOpts) => boolean | undefined;
    remove: (id: string) => boolean;
    setVisible: (id: string, visible: boolean) => boolean;
    setToggetVisible: (id: string) => boolean;
    isVisible: (id: string) => boolean;
    clear: () => boolean;
    stop: (id: string) => boolean;
    start: (id: string) => boolean;
  },
  /**轨迹加载 */
  UseTrajectory: {
    trajectoryLayerMap: React.MutableRefObject<Map<string, OLMAP.trajectoryLayerType>>;
    add: (opts: OLMAP.TrajectoryType.AddTrajectoryOpts) => boolean;
    remove: (id: string) => boolean;
    setVisible: (id: string, visible: boolean) => boolean;
    setToggetVisible: (id: string) => boolean;
    isVisible: (id: string) => boolean;
    clear: () => boolean;
    restart: (id: string) => boolean;
  },
  /**事件绑定 */
  UseEvent: {
    listenerRef: React.MutableRefObject<Record<OLMAP.UseEventType.EventType, Record<string, (e: OLMAP.UseEventType.MapBrowserEventType) => void> | undefined>>;
    /**
     * 添加监听事件
     * @param key 索引
     * @param type 事件类型
     * @param listener 监听回调函数
     */
    addEventListener: (key: string, type: OLMAP.UseEventType.EventType, listener: (e: OLMAP.UseEventType.MapEvent) => void) => void;
    /**
     * 移除监听事件 （key若不传，则清除全部type）
     * @param type 事件类型
     * @param key 索引
     */
    removeEventListener: (type: OLMAP.UseEventType.EventType, key?: string | string[] | undefined) => void;
    /**
     * 清空所有事件，组件卸载时使用，防止事件清空遗漏。
     */
    clearEventListener: () => void;
  },
  /**地图交互 */
  UseInteraction: {
    /**地图交互，点击样式select */
    addSelect: (opts: OLMAP.UseInteractionType.AddInteractionSelectOpts) => boolean | olInteraction.Select;
    /**交互-画线，点，面 */
    addDraw: (opts: OLMAP.UseInteractionType.AddInteractionDrawOpts) => boolean | olInteraction.Draw;
    remove: (interactionKey: string | string[]) => boolean;
    clear: () => boolean;
  }
}

declare namespace OLMAP {
  export namespace LayerType {
    // 地图图层type--------------
    export type TiandituOpts = {
      /**
       * 天地图服务器二级域名
       * @default t0
       */
      domain?: 't0' | 't1' | 't2' | 't3' | 't4' | 't5' | 't6' | 't7' | 't{0-6}';
      /**
       * 天地图服务密钥
       * @description 测试密钥，一天一万次调用，请根据项目申请！！！！！ https://console.tianditu.gov.cn/api/key
       * @default 8abf1ede052e23d2322a54905de565a1
       */
      tk?: string;
      // 灰色蒙版
      greyMask?: boolean;
      //   矢量底图 | 矢量标记 | 影像底图 | 影像标注 | 地形晕眩 | 地形注记 | 全球境界 （c:经纬度投影epsg:EPSG:4326,w:球面墨卡托投影epsg:EPSG:3857）
      type: 'vec_w' | 'cva_w' | 'img_w' | 'cia_w' | 'ter_w' | 'cta_w' | 'ibo_w' | 'vec_c' | 'cva_c' | 'img_c' | 'cia_c' | 'ter_c' | 'cta_c' | 'ibo_c',
    }
    export type TileLayerOpts = {
      /** Opacity (0, 1). */
      opacity?: number;
      /** Visibility */
      visible?: boolean;
      /**
       * 层级
       * @default -1
       */
      zIndex?: number;
      /** The minimum resolution (inclusive) at which this layer will be visible. */
      minResolution?: number;
      maxResolution?: number;
      minZoom?: number;
      maxZoom?: number;
      extent?: any;
      properties?: any;
      source?: XYZSource | WMTSSource;
    } & Record<string, any>;
    export type BaiduLayerOpts = {
      /** Opacity (0, 1). */
      opacity?: number | undefined;
      /** Visibility */
      visible?: boolean | undefined;
      /**
       * 层级
       * @default -1
       */
      zIndex?: number | undefined;
      /** The minimum resolution (inclusive) at which this layer will be visible. */
      minResolution?: number | undefined;
      maxResolution?: number | undefined;
      minZoom?: number | undefined;
      maxZoom?: number | undefined;
      extent?: any;
      properties?: any;
      source?: XYZSource | undefined;
    } & Record<string, any> & {
      tileUrlFunction?: ((e: number[]) => string) | undefined;
      customid?: "normal" | "light" | "dark" | "redalert" | "googlelite" | "grassgreen" | "midnight" | "pink" | "darkgreen" | "bluish" | "grayscale" | "hardedge" | undefined;
    }
    export type GaodeLayerOpts = {
      type?: '道路带注记' | '道路不带注记' | '矢量带标注' | '影像不带注记' | '矢量地图带注记' | '矢量地图不带注记'
    } & Record<string, any>
    export type TileUpYouLayerOpts = {
      /** Opacity (0, 1). */
      opacity?: number | undefined;
      /** Visibility */
      visible?: boolean | undefined;
      /**
       * 层级
       * @default -1
       */
      zIndex?: number | undefined;
      /** The minimum resolution (inclusive) at which this layer will be visible. */
      minResolution?: number | undefined;
      maxResolution?: number | undefined;
      minZoom?: number | undefined;
      maxZoom?: number | undefined;
      extent?: any;
      properties?: any;
      source?: XYZSource | undefined;
    } & Record<string, any> & {
      key: string;
    }
    export type TileArcGISRestLayerOpts = {
      /** Opacity (0, 1). */
      opacity?: number | undefined;
      /** Visibility */
      visible?: boolean | undefined;
      /**
       * 层级
       * @default -1
       */
      zIndex?: number | undefined;
      /** The minimum resolution (inclusive) at which this layer will be visible. */
      minResolution?: number | undefined;
      maxResolution?: number | undefined;
      minZoom?: number | undefined;
      maxZoom?: number | undefined;
      extent?: any;
      properties?: any;
      source?: TileArcGISRestSource | undefined;
    } & Record<string, any> & {
      url: string;
      key: string;
      arcGisRestopts?: OLMAP.SourceType.ArcGisRest;
    }
    export type TileWmtsLayerOpts = {
      /** Opacity (0, 1). */
      opacity?: number | undefined;
      /** Visibility */
      visible?: boolean | undefined;
      /**
       * 层级
       * @default -1
       */
      zIndex?: number | undefined;
      /** The minimum resolution (inclusive) at which this layer will be visible. */
      minResolution?: number | undefined;
      maxResolution?: number | undefined;
      minZoom?: number | undefined;
      maxZoom?: number | undefined;
      extent?: any;
      properties?: any;
      source?: TileArcGISRestSource | undefined;
    } & Record<string, any> & {
      url: string;// wmts地图服务url
      key: string;
      wmtsSourceOpts?: OLMAP.SourceType.WMTS;
    }
  }
  // 边界图层类型
  export namespace AreaFeatureType {
    // 新增（UseAreaFeatures.add）边界参数
    export type AddAreaOpts = {
      /** 组Id */
      groupId: string;
      /**
       * 背景颜色
       * @default rgba(137,119,251,0.5)
       */
      backgroundColor?: string;
      /**
       * 边框大小
       * @default 2
       */
      borderSize?: number;
      /**
       * 边框颜色
       * @default #fff
       */
      borderColor?: string;
      /**
       * 文本显示字段
       * @default name
       */
      textParam?: string;
      /** 自定义样式方法 */
      styleFn?: (icon: Area) => olStyle.Style;
      /** 区域列表 */
      areaList: Area[];
      /**
       * 数据锁
       * @description true为锁定，只允许添加一次，false可覆盖上次
       * @default true
       */
      isLock?: boolean;
      /**
       * 默认可见性
       * @default true
       */
      defaultVisible?: boolean;
      /**
       * 层级
       * @default 10
       */
      zIndex?: number;
    } & TextOpts;
  }
  // 图标图层
  export namespace IconFeatureType {
    export type AddIconOpts = {
      /** 组Id */
      groupId: string;
      /** 图标地址（iconList的src优先级高于外部） */
      iconSrc: string;
      /** 自定义样式方法 */
      styleFn?: (icon: Icon) => olStyle.Style;
      /** 图标列表 */
      iconList: Icon[];
      /**
       * 数据锁
       * @description true为锁定，只允许添加一次，false可覆盖上次
       * @default true
       */
      isLock?: boolean;
      /**
       * 默认可见性
       * @default true
       */
      defaultVisible?: boolean;
      /**
       * 层级
       * @default 20
       */
      zIndex?: number;
    } & TextOpts & IconOpts;
  }
  // 聚合图层
  export namespace ClusterFeatureType {
    export type AddClusterOpts = {
      /** 只有一个的时候展示的图标 */
      onlyOneSrc: string;
      /** 聚合组展示的图标 */
      groupSrc: string;
      /**
       * 聚合范围 m
       * @default 200
       */
      distance?: number;
      /** 自定义样式方法 */
      styleFn?: (feature: RenderFeature | Feature<geom.Geometry>, styleOpts: Omit<OLMAP.ClusterFeatureType.AddClusterOpts, 'groupId' | 'iconList' | 'distance' | 'styleFn'>) => olStyle.Style;
      /** 图标列表 */
      iconList: Icon[];
      /**
       * 数据锁
       * @description true为锁定，只允许添加一次，false可覆盖上次
       * @default true
       */
      isLock?: boolean;
      /**
       * 是否将文本显示聚合图标数量
       * @default false
       */
      isShowPointerNum?: boolean;
      /**
       * 层级
       * @default 20
       */
      zIndex?: number;
    } & Omit<OLMAP.IconFeatureType.AddIconOpts, 'iconList' | 'iconSrc' | 'styleFn'>;
  }
  // 浮动层
  export namespace OverlayFeaturesType {
    /** 新增浮动层配置 */
    export type AddOverlayerOpts = {
      /** 索引 */
      id: string;
      /**
       * html元素
       * @description html元素可以提前配置display:none
       */
      element: HTMLElement | HTMLDivElement;
      /**
       * 设置Overlay显示的坐标点
       * @description undefined隐藏
       */
      position?: [number, number] | number[];
      /**UseInteractionType
       * 设置Overlay在position处的位置
       * @default bottom-center
       */
      positioning?: 'bottom-left' | 'bottom-center' | 'bottom-right' | 'center-left' | 'center-center' | 'center-right' | 'top-left' | 'top-center' | 'top-right';
      /**
       * 阻止事件冒泡
       * @default true
       */
      stopEvent?: boolean;
      /**
       * 偏移量
       * @default [0,0]
       */
      offset?: [number, number];
    };
  }
  // 浮动层切换
  export namespace OverlayChangeType {
    export type OverlayChangeOpts<T = {}> = {
      /**
       * 时间间隔（s）
       * @default 4
       */
      time?: number;
      pointerList: (T & Pointer)[];
      getChangeDataFn?: (pointer: T & Pointer) => void;
    } & Omit<OLMAP.OverlayFeaturesType.AddOverlayerOpts, 'position'>;
  }
  // 运动轨迹
  export namespace TrajectoryType {
    export type AddTrajectoryOpts = {
      /** 索引 */
      id: string;
      /**
       * 边框大小
       * @default 4
       */
      borderSize?: number;
      /**
       * 边框颜色
       * @default #f00
       */
      borderColor?: string;
      /**
       * 插值
       * @description 插值越大越平滑
       * @default 40
       */
      interpolation?: number;
      /** 自定义样式方法 */
      styleFn?: (feature: RenderFeature | Feature<geom.Geometry>) => olStyle.Style;
      /** 实时获取经纬度 */
      getCoordinate?: (coordinate: [number, number]) => void;
      /** 经纬度列表 */
      list: TrajData[];
      /**
       * 数据锁
       * @description true为锁定，只允许添加一次，false可覆盖上次
       * @default true
       */
      isLock?: boolean;
      /**
       * 层级
       * @default 30
       */
      zIndex?: number;
    };
  }
  // 事件方法类型
  export namespace UseEventType {
    export type EventType = 'change' | 'click' | 'singleclick' | 'dblclick' | 'moveend' | 'movestart' | 'pointerdrag' | 'pointermove';
    export type MapEvent = {
      iconFeatures?: OLMAP.UseEventType.FeaturesType[];
      areaFeatures?: OLMAP.UseEventType.FeaturesType[];
      clusterFeatures?: OLMAP.UseEventType.FeaturesType[];
      clickFeatures?: OLMAP.UseEventType.FeaturesType[];
      clickType: 'icon' | 'area' | 'cluster';
      features?: OLMAP.UseEventType.FeaturesType[];
      mapCenter?: number[];
      clickCoordinate?: number;
    } & MapBrowserEvent<any>;
    export type MapBrowserEventType = MapBrowserEvent<any>;
    export type FeaturesType = (RenderFeature | Feature<Geometry>) & { userData?: any };
    /**
     * altKeyOnly 如果只按下 alt 键
     * altShiftKeysOnly 如果仅按下 alt 键和 shift 键，则返回
     * always 始终
     * click 单击
     * doubleClick 双击
     * focus 聚焦
     * mouse Onlyly 鼠标事件
     * never 从不
     * noModifierKeys 如果未按任何修饰键（alt 键、shift 键或平台修饰键）
     * penOnly 事件源自数字笔
     * platformModifierKeyOnly 如果只按下平台修饰键（Mac 上的元键，否则为 ctrl 键）
     * pointerMove 浏览器事件
     * primaryAction 如果事件源自与曲面接触的主指针或按下鼠标左键
     * shiftKeyOnly 如果只按下 shift 键
     * singleClick 地图单击事件
     * targetNotitable 如果目标元素不可编辑
     * touchOnly 事件源自可触摸设备
     */
    export type ConditionType = 'altKeyOnly' | 'altShiftKeysOnly' | 'always' | 'click' | 'doubleClick' | 'focus' | 'mouse' | 'never' | 'noModifierKeys' | 'penOnly' | 'platformModifierKeyOnly' | 'pointerMove' | 'primaryAction' | 'shiftKeyOnly' | 'singleClick' | 'targetNotitable' | 'touchOnly' | string;
  }
  // 地图交互
  export namespace UseInteractionType {
    export type AddInteractionSelectOpts = {
      interactionKey: string,
      layerKeys: string[],
      eventType: "change" | "error" | "select" | "propertychange" | "change:active",
      eventCallback?: (e: any) => void,
      styleFn?: (feature: OLMAP.UseEventType.FeaturesType) => olStyle.Style;
    } & Record<string, any>;
    export type AddInteractionDrawOpts = {
      interactionKey: string;
      style?: olStyle.Style;
      drawType?: 'Polygon' | 'LineString' | 'MultiPolygon' | 'MultiLineString' | 'LinearRing' | 'Point' | 'MultiPoint' | 'GeometryCollection' | 'Circle';
      clickTolerance?: number;// 6px 绘制地图当按下鼠标和抬起鼠标左键之间的这段时间段内没超过多少px，默认添加一个点。相反如果超过了多少px，那么不会添加点，只是平移一下地图
      features?: any;
      snapTolerance?: number;// 12px 最后一点和第一点，吸附重合距离，小于12px,吸附到第一个点位
      maxPoints?: number;// 表示绘制单个要素（面和线）最多的点数限制，默认没有限制。
      minPoints?: number;// 表示绘制单个要素（面和线）需要的最少点数，面默认为 3，线默认为 2
      geometryFunction?: (coordinates: any, geometry: OLMAP.GeometryType) => OLMAP.GeometryType
      geometryName?: string;// 绘制的 geometry 的名称
      condition?: OLMAP.UseEventType.ConditionType;//规定了什么情况下触发 draw 操作，默认不需要特殊条件进行触发。一个函数，接受一个ol.MapBrowserEvent类型的参数，返回一个布尔值表示是否应该调用事件处理函数。默认情况下，增加一个顶点，类型为 ol.events.ConditionType
      eventList?: ({
        eventType?: "change" | "error" | "propertychange" | "change:active" | "drawstart" | "drawend" | "drawabort";
        eventCallback?: (e: any) => any
      })[];
      // 在修改或绘制矢量要素时处理矢量要素的捕捉 只要之前添加即可
      snapOpts?: {
        isAdd?: boolean;
        features?: any;
        edge?: boolean;// 默认为 true 与边缘对齐。
        vertex?: boolean;// 默认为 true 与顶点对齐
        pixelTolerance?: number; // 默认为 10
        source?: any;
        eventList?: ({
          eventType?: "change" | "error" | "propertychange" | "change:active";
          eventCallback?: (e: any) => any
        })[];
      } & Record<string, any>;
      modifyOpts?: {
        isAdd?: boolean;
        condition?: any;// 一个函数 指示是否将该事件视为向草绘添加顶点或移动 默认值为ol/events/condition.primaryAction.
        deleteCondition?: any;// 一个函数 指示是否应处理该事件 默认情况下，module:ol/events/condition.singleClick跟module:ol/events/condition.altKeyOnly导致顶点删除。
        insertVertexCondition?: any;// 一个函数，它接受module:ol/MapBrowserEvent~MapBrowserEvent并返回一个布尔值，以指示是否应将新折点添加到草绘要素。默认值为module:ol/events/condition.always.
        pixelTolerance?: number;// 10 像素容差，用于考虑指针足够接近线段或顶点以进行编辑
        style?: olStyle.Style;
        source?: any;
        hitDetection?: boolean;//配置后，将考虑根据其视觉外观修改点要素，而不是位于指针位置内。当pixelTolerancemodule:ol/layer/BaseVector~BaseVectorLayer，则仅考虑该图层上要素的渲染制图表达。
        features?: any;
        wrapX?: boolean;//false 在草图叠加层上水平环绕世界
        snapToPointer?: boolean;// 默认！hitDetection 在 中单击时，正在修改的顶点、点或线段会捕捉到指针坐标。
        eventList?: ({
          eventType?: "change" | "error" | "propertychange" | "change:active" | "modifystart" | "modifyend";
          eventCallback?: (e: any) => any
        })[];
      } & Record<string, any>;
      source?: any;
    } & Record<string, any>;
  }
  // --------------
  // 切片类型
  export type TileType<T extends TileSource> = layer.Tile<T>;
  // source类型
  export type SourceAllType = source.WMTS | source.XYZ | source.TileArcGISRest;
  // 区域，边界图层类型
  export type areaMapLayerType = layer.Vector<source.Vector<geom.Geometry | geom.Circle | geom.GeometryCollection | geom.Polygon & Record<string, any>>>;
  // 点位图层类型
  export type iconFeatureLayerType = layer.Vector<source.Vector<geom.Geometry | geom.Circle | geom.GeometryCollection | geom.Polygon | geom.Point>>;
  // 聚合图层类型
  export type clusterFeatureLayerType = layer.Vector<source.Cluster>;
  // 浮动层类型
  export type OverlayLayerType = Overlay;
  // 轨迹图层
  export type trajectoryLayerType = layer.Vector<source.Vector<geom.Geometry | geom.Circle | geom.GeometryCollection | geom.Polygon & Record<string, any>>>;
  // 勾绘的 geometry 类型
  export type GeometryType = geom.Polygon | geom.MultiPolygon | geom.LineString | geom.Point | geom.MultiPoint | geom.GeometryCollection | geom.LinearRing;

  // 数据源参数
  export namespace SourceType {
    export type ArcGisRest = {
      params?: { [key: string]: any };//ArcGIS Rest 参数
      hidpi?: boolean; // 默认true  //从远程服务器请求映像时使用该值
      crossOrigin?: string | undefined;// 常用：anonymous 加载图像的属性。请注意，如果要使用 Canvas 渲染器访问像素数据，则必须提供一个值。看crossOrigincrossOriginhttps://developer.mozilla.org/en-US/docs/Web/HTML/CORS_enabled_image了解更多详情。
      wrapX?: boolean;// 默认true 是否横向包装世界。
    } & Record<string, any> & TileImageBase;
    export type WMTS = {
      layer?: string;
      matrixSet?: string;
      format?: string;
      projection?: any;// 坐标系
      tileGrid?: any;
      style?: string;
      wrapX?: boolean;
    } & source.WMTS & TileImageBase;
    export type XYZ = {

    } & source.XYZ & TileImageBase;
    export type TileImageBase = {
      attributions?: any;
      attributionsCollapsible?: boolean;// (defaults to true) 属性是可折叠的
      cacheSize?: number;// 初始切片缓存大小。将自动增长以至少容纳视区中的切片数
      crossOrigin?: string | undefined;// 常用：anonymous 加载图像的属性。请注意，如果要使用 Canvas 渲染器访问像素数据，则必须提供一个值。看crossOrigincrossOriginhttps://developer.mozilla.org/en-US/docs/Web/HTML/CORS_enabled_image了解更多详情。
      imageSmoothing?: boolean; // 默认true  废弃 使用interpolate
      interpolate?: boolean; // 默认true 重新取样时使用插值。默认情况下，重新采样时使用线性插值。设置为 false 以改用最近的邻居。
      opaque?: boolean;// default false 图层是否不透明
      // params?: { [key: string]: any };//ArcGIS Rest 参数
      // hidpi?: boolean; // 默认true  //从远程服务器请求映像时使用该值
      projection?: any;//投影。默认值为视图投影。投影代码必须包含以 分隔的数字结束部分，或者整个代码必须形成有效的 ArcGIS 空间参考定义。
      reprojectionErrorThreshold?: number;// 默认0.5  //允许的最大重新投影误差（以像素为单位）。较高的值可以提高重新投影性能，但会降低精度
      state?: any;// 源状态
      tileClass?: any;//	用于实例化图像磁贴的类。默认值为module:ol/ImageTile~ImageTile.
      tileGrid?: any;// 磁贴网格。基于服务器支持的分辨率、切片大小和范围。如果未定义，则将使用默认格网：如果存在投影范围，则格网将基于该范围;否则，将使用基于原点为 0，0 的全局范围的格网。
      tileLoadFunction?: any;// 加载给定 URL 的磁贴的可选函数
      tilePixelRatio?: number;//	数字（默认为 1）	切片服务使用的像素比率。例如，如果磁贴服务将 256px x 256px 的磁贴进行广告宣传，但实际上发送了 512px x 512px 的图像（对于视网膜/hidpi 设备），则应将其设置为 。tilePixelRatio:2
      tileUrlFunction?: any;//	模块：ol/Tile~UrlFunction|定义	在给定图块坐标和投影的情况下获取图块 URL 的可选函数。
      url?: string;//地图服务或影像服务的 ArcGIS Rest 服务 URL。url 应包含 /MapServer 或 /ImageServer。
      wrapX?: boolean;// 默认true 是否横向包装世界。
      transition?: number;// 用于渲染的不透明度过渡的持续时间。要禁用不透明度过渡，请传递 。transition:0
      urls?: string[];// ArcGIS Rest service urls.当 ArcGIS 服务支持用于导出请求的多个 URL 时，请使用此选项
      key?: string;//	字符串|定义	用于正确提取缓存的可选切片键
      zDirection?: number;// 选择在整数缩放级别之间是使用具有较高或较低缩放级别的磁贴
    }
  }
}

// 轨迹数据
declare type TrajData = {
  lat?: number;
  lng?: number;
} & Record<string, any>;

// 点位数据
declare type Pointer = {
  lng?: number;
  lat?: number;
} & Record<string, any>;

/** 区域对象 */
declare type Area = {
  lng?: number;
  lat?: number;
  wkt?: string;
} & Omit<OLMAP.AreaFeatureType.AddAreaOpts, 'groupId' | 'textParam' | 'styleFn' | 'areaList'> & Record<string, any>;

declare type Icon = {
  lng?: number;
  lat?: number;
  iconSrc?: string;
} & Omit<OLMAP.IconFeatureType.AddIconOpts, 'iconSrc' | 'groupId' | 'textParam' | 'styleFn' | 'iconList' | 'defaultVisible'> & Record<string, any>;

// 图标参数
export declare type IconOpts = {
  /**
   * imgsource
   */
  image?: ImageStyle;
  /**
   * 偏移量
   * @default [0.5,30]
   */
  iconAnchor?: [number, number];
  /**
   * 原点位置，取值为bottom-left, bottom-right, top-left 或 top-right
   * @default top-right
   */
  iconAnchorOrigin?: 'bottom-left' | 'bottom-right' | 'top-left' | 'top-right';
  /**
   * 透明度
   * @default 1
   */
  iconOpacity?: number;
  /**
   * 缩放
   * @default 0.4
   */
  iconScale?: number;
};

// 文字参数
export declare type TextOpts = {
  /**
   * 文字可见性
   * @default true
   */
  textVisible?: boolean;
  /**
   * 文本显示字段
   * @default name
   */
  textParam?: string;
  /**
   * 文字偏移量
   * @default [20,0]
   */
  textOffset?: [number, number];
  /**
   * 取值“ left”，“ right”，“ center”，“ end”或“ start”。
   * @default left
   */
  textAlign?: 'left' | 'right' | 'center' | 'end' | 'start';
  /**
   * 文本的基准线 取值范围: ‘bottom’, ‘top’, ‘middle’, ‘alphabetic’, ‘hanging’, ‘ideographic’.
   * @default middle
   */
  textBaseline?: 'bottom' | 'top' | 'middle' | 'alphabetic' | 'hanging' | 'ideographic';
  /**
   * 字体样式
   * @default '12px PingFang'
   */
  font?: string;
  /**
   * 文本颜色
   * @default #fff
   */
  textColor?: string;
  /**
   * 文本背景颜色
   * @default rgba(0,0,0,.5)
   */
  textBackgroundColor?: string;
  /**
   * 内边距
   * @default [3,8,3,8]
   */
  textPadding?: [number, number, number, number];
};

export declare type Keys<T> = T extends object ? {
  [K in keyof T]-?: K;
}[keyof T] : never;

export declare type TOOLSTYPE = {
  FullScreen: FullScreen;
  Attribution: Attribution;
  MousePosition: MousePosition;
  OverviewMap: OverviewMap;
  Rotate: Rotate;
  ScaleLine: ScaleLine;
  Zoom: Zoom;
  ZoomSlider: ZoomSlider;
  ZoomToExtent: ZoomToExtent;
};
