/* eslint-disable @typescript-eslint/switch-exhaustiveness-check */
/* eslint-disable prefer-const */
/*
 * @Author: zhenxing xiezhenxing@codvision.com
 * @Date: 2022-05-23 17:05:53
 * @LastEditors: zhenxing xiezhenxing@codvision.com
 * @LastEditTime: 2022-08-29 11:08:31
 * @FilePath: \组件库\src\pages\MapComponent\OlTest\components\OlMap\index.tsx
 * @Description: openlayers二次封装组件
 */
import * as React from 'react';
import {
  memo,
  forwardRef,
  useImperativeHandle,
  useRef,
  useEffect,
} from 'react';
import * as ol from 'ol';
import * as proj from 'ol/proj';
import * as extent from 'ol/extent';
import * as layer from 'ol/layer';
import * as source from 'ol/source';
import * as geom from 'ol/geom';
import * as olStyle from 'ol/style';
import * as olInteraction from 'ol/interaction';
import tilegridWMTS from 'ol/tilegrid/WMTS';
import { MultiPolygon, Polygon } from 'ol/geom';
import Feature from 'ol/Feature';
import type { MapOptions } from 'ol/PluggableMap';
// import {
//   Area,
//   Icon,
//   OLMAP,
//   Keys,
//   TOOLSTYPE,
//   OlMyMapProps,
//   OlMyMapRef,
// } from './index.d';
import baiduTile from './baiduTile';
import TileGrid from 'ol/tilegrid/TileGrid';
import * as WKTTOOLS from 'terraformer-wkt-parser';
import WKT from 'ol/format/WKT';
import * as loadshGet from 'lodash';
// import RenderFeature from 'ol/render/Feature';
import {
  DEFAULTTEXT,
  DrawDefaultStyle,
  TOOLSDATA,
} from './mapTools/utils/config';
import getpolyArea from './mapTools/getpolyArea';
import VectorSource from 'ol/source/Vector';
import VectorLayer from 'ol/layer/Vector';
import UseFormatArea from './mapTools/UseFormatArea/index';
import UseCoordinate from './mapTools/UseCoordinate/index';
import UseCoordinateType from './mapTools/UseCoordinate/index.d';
import UseFormatAreaType from './mapTools/UseFormatArea/index.d';
import WMTSTileGrid from 'ol/tilegrid/WMTS';
import filterLngLatByList from './mapTools/utils/filterLngLatByList';
import getParams from './mapTools/utils/getParams';
import setFocus from './mapTools/utils/setFocus';
import createIconStyle from './mapTools/utils/createIconStyle';
import createClusterStyle from './mapTools/utils/createClusterStyle';
import createLineStyle from './mapTools/utils/createLineStyle';
import coordinateMoreChildren from './mapTools/utils/coordinateMoreChildren';
import * as GetId from './mapTools/getGroupId';
import Attribution from 'ol/control/Attribution';
import FullScreen from 'ol/control/FullScreen';
import MousePosition from 'ol/control/MousePosition';
import OverviewMap from 'ol/control/OverviewMap';
import Rotate from 'ol/control/Rotate';
import ScaleLine from 'ol/control/ScaleLine';
import Zoom from 'ol/control/Zoom';
import ZoomSlider from 'ol/control/ZoomSlider';
import ZoomToExtent from 'ol/control/ZoomToExtent';
import XYZSource from 'ol/source/XYZ';
import WMTSSource from 'ol/source/WMTS';
import TileArcGISRestSource from 'ol/source/TileArcGISRest';
import Overlay from 'ol/Overlay';
import RenderFeature from 'ol/render/Feature';
import Geometry from 'ol/geom/Geometry';
import MapBrowserEvent from 'ol/MapBrowserEvent';
import ImageStyle from 'ol/style/Image';
import TileSource from 'ol/source/Tile';

const { projzh, bmercResolutions } = baiduTile;

/** 地图props类型  */
export type OlMyMapProps = {
  mapId: string;
  center?: number[];
  zoom?: number;
  minZoom?: number;
  maxZoom?: number;
  xScale?: number;
  yScale?: number;
  // 默认图层加载
  defaultLayersVisible?: boolean;
  otherMapOpts?: MapOptions;
  children?: any;
} & Record<string, any>;

/** refProps */
export type OlMyMapRef = {
  /**实例化ol.map对象ref */
  BaseMapRef: {
    mapRef: React.MutableRefObject<ol.Map | undefined>;
  };
  /** 地图所有类型图层(除交互事件图层外)统一操作 */
  UseAllMapLayer: {
    /** 图层显示与隐藏 */
    setLayerVisible: (key: string | string[], visible: boolean) => boolean;
    /** 图层显隐切换 */
    setLayerToggetVisible: (key: string | string[]) => boolean;
    /** 移除图层 */
    removeLayer: (key: string | string[]) => boolean;
  },
  /**地图控制器配置 */
  UseControls: {
    /**
     * 添加控制按钮
     * @param control
     */
    add: (control: Keys<TOOLSTYPE> | Keys<TOOLSTYPE>[]) => void;
    /**
     * 删除控制按钮
     * @param control
     */
    remove: (control: Keys<TOOLSTYPE> | Keys<TOOLSTYPE>[]) => void;
    /**
     * 清空
     */
    clear: () => void;
  },
  /**样式设置 */
  UseStyle: {
    createClusterStyle: (feature: RenderFeature | Feature<geom.Geometry>, styleOpts: Omit<OLMAP.ClusterFeatureType.AddClusterOpts, 'groupId' | 'iconList' | 'distance' | 'styleFn'>) => olStyle.Style,
    createAreaStyle: (feature: {
      text: string;
    } & Pick<OLMAP.AreaFeatureType.AddAreaOpts, "textVisible" | "textParam" | "textOffset" | "textAlign" | "textBaseline" | "font" | "textColor" | "textBackgroundColor" | "textPadding" | "zIndex" | "styleFn" | "isLock" | "defaultVisible" | "backgroundColor" | "borderSize" | "borderColor">) => olStyle.Style,
    createIconStyle: (styleOpts: {
      text: string;
    } & Omit<OLMAP.IconFeatureType.AddIconOpts, 'groupId' | 'iconList'>) => olStyle.Style,
    createLineStyle: (styleOpts: Omit<OLMAP.TrajectoryType.AddTrajectoryOpts, 'id' | 'list'>) => olStyle.Style,
  };
  /**区域处理工具 */
  UseFormatArea: UseFormatAreaType;
  /**点位处理工具 */
  UseCoordinate: UseCoordinateType;
  /**视图处理工具 */
  UseView: {
    setCenter: (coordinate: [number, number]) => boolean;
    setZoom: (zoom: number) => boolean;
    /**
     * 通过wkt/geojson 自适应可视区域
     * @param str wkt/geojson 字符串
     * @param padding [top,right,bottom,left]
     */
    setFit: (str: string, padding: [number, number, number, number]) => boolean;
    getCenter: () => number[] | undefined;
    getZoom: () => number | undefined;
  },
  /**地图底图加载 */
  UseTileLayer: {
    /**地图底图图层ref---- new Map() */
    mapBaseLayerRef: React.MutableRefObject<Map<string, OLMAP.TileType<OLMAP.SourceAllType>>>;
    /**设置天地图矢量底图 */
    setTDTLayer: (options: OLMAP.LayerType.TileLayerOpts & OLMAP.LayerType.TiandituOpts) => boolean,
    /**设置天地图路网 */
    setTDTLWLayer: (options: OLMAP.LayerType.TileLayerOpts & Omit<OLMAP.LayerType.TiandituOpts, 'type'>) => boolean,
    /**设置百度地图 */
    setBaiduLayer: (options: OLMAP.LayerType.TileLayerOpts) => boolean,
    /**设置高德地图 */
    setGaoDeLayer: (options: OLMAP.LayerType.TileLayerOpts & OLMAP.LayerType.GaodeLayerOpts) => boolean,
    /**
     * 添加影像切片图层
     * @param options TileLayer配置项
     */
    setYXLayer: (options: OLMAP.LayerType.TileLayerOpts & { access_token?: string, curl?: string, sku?: string }) => boolean,
    /**设置自定义地图 */
    setUpYouLayer: (options: OLMAP.LayerType.TileUpYouLayerOpts) => boolean,
    /**设置arcgisrect */
    setTileArcGISRestLayer: (options: OLMAP.LayerType.TileArcGISRestLayerOpts) => boolean,
    /**设置wmts图层 */
    setWmtsLayer: (options: OLMAP.LayerType.TileWmtsLayerOpts) => boolean,
    /**移除图层 */
    removeLayer: (key: string | string[]) => boolean,
    /**设置图层visible */
    setVisible: (classname: string | string[], visible: boolean) => boolean
    setToggetVisible: (classname: string | string[]) => boolean,
    isVisible: (classname: string | string[]) => boolean,
    /**隐藏所有的底图图层 */
    hideAll: () => boolean;
    /**显示所有的底图图层 */
    showAll: () => boolean;
    /**清楚图层 */
    clear: () => boolean;
  },
  /**边界图层 */
  UseAreaFeatures: {
    /**地图边界图层ref---- new Map() */
    areaMapLayerRef: React.MutableRefObject<Map<string, OLMAP.areaMapLayerType>>;
    add: (opts: OLMAP.AreaFeatureType.AddAreaOpts) => false | Feature<any>[];
    /**移除图层 */
    remove: (groupId: string | string[]) => boolean,
    setVisible: (groupId: string | string[], visible: boolean) => boolean;
    isVisible: (groupId: string | string[]) => boolean;
    setToggetVisible: (groupId: string | string[]) => boolean;
    clear: () => boolean;
  },
  /** 图标图层 */
  UseIconFeatures: {
    /**地图点位图层ref---- new Map() */
    iconFeatureLayerRef: React.MutableRefObject<Map<string, OLMAP.iconFeatureLayerType>>;
    add: (opts: OLMAP.IconFeatureType.AddIconOpts) => false | Feature<geom.Point>[];
    remove: (groupId: string | string[]) => boolean;
    setVisible: (groupId: string | string[], visible: boolean) => boolean;
    setToggetVisible: (groupId: string | string[]) => boolean;
    isVisible: (groupId: string | string[]) => boolean;
    clear: () => boolean;
    /**
     * 修改位置
     * @param groupId 组id
     * @param find 类似 list.find
     * @param coordinate 经纬度
     */
    setPosition: (groupId: string, find: (data: {
      userData: Icon;
    }) => boolean, coordinate: number[]) => any;
    focus: (groupId: string | string[], padding?: [number, number, number, number]) => boolean | undefined;
    check: (groupId: string) => boolean;
  },
  /**聚合点位加载 */
  UseClusterFeatures: {
    /**地图聚合点位图层ref---- new Map() */
    clusterFeatureLayerRef: React.MutableRefObject<Map<string, OLMAP.clusterFeatureLayerType>>;
    add: (opts: OLMAP.ClusterFeatureType.AddClusterOpts) => false | Feature<geom.Point>[];
    remove: (groupId: string | string[]) => boolean;
    setVisible: (groupId: string | string[], visible: boolean) => boolean;
    setToggetVisible: (groupId: string | string[]) => boolean;
    isVisible: (groupId: string | string[]) => boolean;
    clear: () => boolean;
    focus: (groupId: string | string[], padding?: [number, number, number, number]) => boolean | undefined;
    check: (groupId: string) => boolean;
  },
  /**覆盖层加载 */
  UseOverlayFeatures: {
    positionMap: React.MutableRefObject<Map<string, number[] | undefined>>;
    add: (opts: OLMAP.OverlayFeaturesType.AddOverlayerOpts) => false | OLMAP.OverlayLayerType;
    remove: (id: string) => boolean;
    setVisible: (id: string, visible: boolean) => boolean;
    setToggetVisible: (id: string) => boolean;
    isVisible: (id: string) => boolean;
    clear: () => boolean;
    setPosition: (id: string, coordinate: [number, number] | number[]) => boolean;
  },
  /**覆盖层定时轮播加载 */
  UseOverlayChange: {
    add: (opts: OLMAP.OverlayChangeType.OverlayChangeOpts) => boolean | undefined;
    remove: (id: string) => boolean;
    setVisible: (id: string, visible: boolean) => boolean;
    setToggetVisible: (id: string) => boolean;
    isVisible: (id: string) => boolean;
    clear: () => boolean;
    stop: (id: string) => boolean;
    start: (id: string) => boolean;
  },
  /**轨迹加载 */
  UseTrajectory: {
    trajectoryLayerMap: React.MutableRefObject<Map<string, OLMAP.trajectoryLayerType>>;
    add: (opts: OLMAP.TrajectoryType.AddTrajectoryOpts) => boolean;
    remove: (id: string) => boolean;
    setVisible: (id: string, visible: boolean) => boolean;
    setToggetVisible: (id: string) => boolean;
    isVisible: (id: string) => boolean;
    clear: () => boolean;
    restart: (id: string) => boolean;
  },
  /**事件绑定 */
  UseEvent: {
    listenerRef: React.MutableRefObject<Record<OLMAP.UseEventType.EventType, Record<string, (e: OLMAP.UseEventType.MapBrowserEventType) => void> | undefined>>;
    /**
     * 添加监听事件
     * @param key 索引
     * @param type 事件类型
     * @param listener 监听回调函数
     */
    addEventListener: (key: string, type: OLMAP.UseEventType.EventType, listener: (e: OLMAP.UseEventType.MapEvent) => void) => void;
    /**
     * 移除监听事件 （key若不传，则清除全部type）
     * @param type 事件类型
     * @param key 索引
     */
    removeEventListener: (type: OLMAP.UseEventType.EventType, key?: string | string[] | undefined) => void;
    /**
     * 清空所有事件，组件卸载时使用，防止事件清空遗漏。
     */
    clearEventListener: () => void;
  },
  /**地图交互 */
  UseInteraction: {
    /**地图交互，点击样式select */
    addSelect: (opts: OLMAP.UseInteractionType.AddInteractionSelectOpts) => boolean | olInteraction.Select;
    /**交互-画线，点，面 */
    addDraw: (opts: OLMAP.UseInteractionType.AddInteractionDrawOpts) => boolean | olInteraction.Draw;
    remove: (interactionKey: string | string[]) => boolean;
    clear: () => boolean;
  }
}

export namespace OLMAP {
  export namespace LayerType {
    // 地图图层type--------------
    export type TiandituOpts = {
      /**
       * 天地图服务器二级域名
       * @default t0
       */
      domain?: 't0' | 't1' | 't2' | 't3' | 't4' | 't5' | 't6' | 't7' | 't{0-6}';
      /**
       * 天地图服务密钥
       * @description 测试密钥，一天一万次调用，请根据项目申请！！！！！ https://console.tianditu.gov.cn/api/key
       * @default 8abf1ede052e23d2322a54905de565a1
       */
      tk?: string;
      // 灰色蒙版
      greyMask?: boolean;
      //   矢量底图 | 矢量标记 | 影像底图 | 影像标注 | 地形晕眩 | 地形注记 | 全球境界 （c:经纬度投影epsg:EPSG:4326,w:球面墨卡托投影epsg:EPSG:3857）
      type: 'vec_w' | 'cva_w' | 'img_w' | 'cia_w' | 'ter_w' | 'cta_w' | 'ibo_w' | 'vec_c' | 'cva_c' | 'img_c' | 'cia_c' | 'ter_c' | 'cta_c' | 'ibo_c',
    }
    export type TileLayerOpts = {
      /** Opacity (0, 1). */
      opacity?: number;
      /** Visibility */
      visible?: boolean;
      /**
       * 层级
       * @default -1
       */
      zIndex?: number;
      /** The minimum resolution (inclusive) at which this layer will be visible. */
      minResolution?: number;
      maxResolution?: number;
      minZoom?: number;
      maxZoom?: number;
      extent?: any;
      properties?: any;
      source?: XYZSource | WMTSSource;
    } & Record<string, any>;
    export type BaiduLayerOpts = {
      /** Opacity (0, 1). */
      opacity?: number | undefined;
      /** Visibility */
      visible?: boolean | undefined;
      /**
       * 层级
       * @default -1
       */
      zIndex?: number | undefined;
      /** The minimum resolution (inclusive) at which this layer will be visible. */
      minResolution?: number | undefined;
      maxResolution?: number | undefined;
      minZoom?: number | undefined;
      maxZoom?: number | undefined;
      extent?: any;
      properties?: any;
      source?: XYZSource | undefined;
    } & Record<string, any> & {
      tileUrlFunction?: ((e: number[]) => string) | undefined;
      customid?: "normal" | "light" | "dark" | "redalert" | "googlelite" | "grassgreen" | "midnight" | "pink" | "darkgreen" | "bluish" | "grayscale" | "hardedge" | undefined;
    }
    export type GaodeLayerOpts = {
      type?: '道路带注记' | '道路不带注记' | '矢量带标注' | '影像不带注记' | '矢量地图带注记' | '矢量地图不带注记'
    } & Record<string, any>
    export type TileUpYouLayerOpts = {
      /** Opacity (0, 1). */
      opacity?: number | undefined;
      /** Visibility */
      visible?: boolean | undefined;
      /**
       * 层级
       * @default -1
       */
      zIndex?: number | undefined;
      /** The minimum resolution (inclusive) at which this layer will be visible. */
      minResolution?: number | undefined;
      maxResolution?: number | undefined;
      minZoom?: number | undefined;
      maxZoom?: number | undefined;
      extent?: any;
      properties?: any;
      source?: XYZSource | undefined;
    } & Record<string, any> & {
      key: string;
    }
    export type TileArcGISRestLayerOpts = {
      /** Opacity (0, 1). */
      opacity?: number | undefined;
      /** Visibility */
      visible?: boolean | undefined;
      /**
       * 层级
       * @default -1
       */
      zIndex?: number | undefined;
      /** The minimum resolution (inclusive) at which this layer will be visible. */
      minResolution?: number | undefined;
      maxResolution?: number | undefined;
      minZoom?: number | undefined;
      maxZoom?: number | undefined;
      extent?: any;
      properties?: any;
      source?: TileArcGISRestSource | undefined;
    } & Record<string, any> & {
      url: string;
      key: string;
      arcGisRestopts?: OLMAP.SourceType.ArcGisRest;
    }
    export type TileWmtsLayerOpts = {
      /** Opacity (0, 1). */
      opacity?: number | undefined;
      /** Visibility */
      visible?: boolean | undefined;
      /**
       * 层级
       * @default -1
       */
      zIndex?: number | undefined;
      /** The minimum resolution (inclusive) at which this layer will be visible. */
      minResolution?: number | undefined;
      maxResolution?: number | undefined;
      minZoom?: number | undefined;
      maxZoom?: number | undefined;
      extent?: any;
      properties?: any;
      source?: TileArcGISRestSource | undefined;
    } & Record<string, any> & {
      url: string;// wmts地图服务url
      key: string;
      wmtsSourceOpts?: OLMAP.SourceType.WMTS;
    }
  }
  // 边界图层类型
  export namespace AreaFeatureType {
    // 新增（UseAreaFeatures.add）边界参数
    export type AddAreaOpts = {
      /** 组Id */
      groupId: string;
      /**
       * 背景颜色
       * @default rgba(137,119,251,0.5)
       */
      backgroundColor?: string;
      /**
       * 边框大小
       * @default 2
       */
      borderSize?: number;
      /**
       * 边框颜色
       * @default #fff
       */
      borderColor?: string;
      /**
       * 文本显示字段
       * @default name
       */
      textParam?: string;
      /** 自定义样式方法 */
      styleFn?: (icon: Area) => olStyle.Style;
      /** 区域列表 */
      areaList: Area[];
      /**
       * 数据锁
       * @description true为锁定，只允许添加一次，false可覆盖上次
       * @default true
       */
      isLock?: boolean;
      /**
       * 默认可见性
       * @default true
       */
      defaultVisible?: boolean;
      /**
       * 层级
       * @default 10
       */
      zIndex?: number;
    } & TextOpts;
  }
  // 图标图层
  export namespace IconFeatureType {
    export type AddIconOpts = {
      /** 组Id */
      groupId: string;
      /** 图标地址（iconList的src优先级高于外部） */
      iconSrc: string;
      /** 自定义样式方法 */
      styleFn?: (icon: Icon) => olStyle.Style;
      /** 图标列表 */
      iconList: Icon[];
      /**
       * 数据锁
       * @description true为锁定，只允许添加一次，false可覆盖上次
       * @default true
       */
      isLock?: boolean;
      /**
       * 默认可见性
       * @default true
       */
      defaultVisible?: boolean;
      /**
       * 层级
       * @default 20
       */
      zIndex?: number;
    } & TextOpts & IconOpts;
  }
  // 聚合图层
  export namespace ClusterFeatureType {
    export type AddClusterOpts = {
      /** 只有一个的时候展示的图标 */
      onlyOneSrc: string;
      /** 聚合组展示的图标 */
      groupSrc: string;
      /**
       * 聚合范围 m
       * @default 200
       */
      distance?: number;
      /** 自定义样式方法 */
      styleFn?: (feature: RenderFeature | Feature<geom.Geometry>, styleOpts: Omit<OLMAP.ClusterFeatureType.AddClusterOpts, 'groupId' | 'iconList' | 'distance' | 'styleFn'>) => olStyle.Style;
      /** 图标列表 */
      iconList: Icon[];
      /**
       * 数据锁
       * @description true为锁定，只允许添加一次，false可覆盖上次
       * @default true
       */
      isLock?: boolean;
      /**
       * 是否将文本显示聚合图标数量
       * @default false
       */
      isShowPointerNum?: boolean;
      /**
       * 层级
       * @default 20
       */
      zIndex?: number;
    } & Omit<OLMAP.IconFeatureType.AddIconOpts, 'iconList' | 'iconSrc' | 'styleFn'>;
  }
  // 浮动层
  export namespace OverlayFeaturesType {
    /** 新增浮动层配置 */
    export type AddOverlayerOpts = {
      /** 索引 */
      id: string;
      /**
       * html元素
       * @description html元素可以提前配置display:none
       */
      element: HTMLElement | HTMLDivElement;
      /**
       * 设置Overlay显示的坐标点
       * @description undefined隐藏
       */
      position?: [number, number] | number[];
      /**UseInteractionType
       * 设置Overlay在position处的位置
       * @default bottom-center
       */
      positioning?: 'bottom-left' | 'bottom-center' | 'bottom-right' | 'center-left' | 'center-center' | 'center-right' | 'top-left' | 'top-center' | 'top-right';
      /**
       * 阻止事件冒泡
       * @default true
       */
      stopEvent?: boolean;
      /**
       * 偏移量
       * @default [0,0]
       */
      offset?: [number, number];
    };
  }
  // 浮动层切换
  export namespace OverlayChangeType {
    export type OverlayChangeOpts<T = {}> = {
      /**
       * 时间间隔（s）
       * @default 4
       */
      time?: number;
      pointerList: (T & Pointer)[];
      getChangeDataFn?: (pointer: T & Pointer) => void;
    } & Omit<OLMAP.OverlayFeaturesType.AddOverlayerOpts, 'position'>;
  }
  // 运动轨迹
  export namespace TrajectoryType {
    export type AddTrajectoryOpts = {
      /** 索引 */
      id: string;
      /**
       * 边框大小
       * @default 4
       */
      borderSize?: number;
      /**
       * 边框颜色
       * @default #f00
       */
      borderColor?: string;
      /**
       * 插值
       * @description 插值越大越平滑
       * @default 40
       */
      interpolation?: number;
      /** 自定义样式方法 */
      styleFn?: (feature: RenderFeature | Feature<geom.Geometry>) => olStyle.Style;
      /** 实时获取经纬度 */
      getCoordinate?: (coordinate: [number, number]) => void;
      /** 经纬度列表 */
      list: TrajData[];
      /**
       * 数据锁
       * @description true为锁定，只允许添加一次，false可覆盖上次
       * @default true
       */
      isLock?: boolean;
      /**
       * 层级
       * @default 30
       */
      zIndex?: number;
    };
  }
  // 事件方法类型
  export namespace UseEventType {
    export type EventType = 'change' | 'click' | 'singleclick' | 'dblclick' | 'moveend' | 'movestart' | 'pointerdrag' | 'pointermove';
    export type MapEvent = {
      iconFeatures?: OLMAP.UseEventType.FeaturesType[];
      areaFeatures?: OLMAP.UseEventType.FeaturesType[];
      clusterFeatures?: OLMAP.UseEventType.FeaturesType[];
      clickFeatures?: OLMAP.UseEventType.FeaturesType[];
      clickType: 'icon' | 'area' | 'cluster';
      features?: OLMAP.UseEventType.FeaturesType[];
      mapCenter?: number[];
      clickCoordinate?: number;
    } & MapBrowserEvent<any>;
    export type MapBrowserEventType = MapBrowserEvent<any>;
    export type FeaturesType = (RenderFeature | Feature<Geometry>) & { userData?: any };
    /**
     * altKeyOnly 如果只按下 alt 键
     * altShiftKeysOnly 如果仅按下 alt 键和 shift 键，则返回
     * always 始终
     * click 单击
     * doubleClick 双击
     * focus 聚焦
     * mouse Onlyly 鼠标事件
     * never 从不
     * noModifierKeys 如果未按任何修饰键（alt 键、shift 键或平台修饰键）
     * penOnly 事件源自数字笔
     * platformModifierKeyOnly 如果只按下平台修饰键（Mac 上的元键，否则为 ctrl 键）
     * pointerMove 浏览器事件
     * primaryAction 如果事件源自与曲面接触的主指针或按下鼠标左键
     * shiftKeyOnly 如果只按下 shift 键
     * singleClick 地图单击事件
     * targetNotitable 如果目标元素不可编辑
     * touchOnly 事件源自可触摸设备
     */
    export type ConditionType = 'altKeyOnly' | 'altShiftKeysOnly' | 'always' | 'click' | 'doubleClick' | 'focus' | 'mouse' | 'never' | 'noModifierKeys' | 'penOnly' | 'platformModifierKeyOnly' | 'pointerMove' | 'primaryAction' | 'shiftKeyOnly' | 'singleClick' | 'targetNotitable' | 'touchOnly' | string;
  }
  // 地图交互
  export namespace UseInteractionType {
    export type AddInteractionSelectOpts = {
      interactionKey: string,
      layerKeys: string[],
      eventType: "change" | "error" | "select" | "propertychange" | "change:active",
      eventCallback?: (e: any) => void,
      styleFn?: (feature: OLMAP.UseEventType.FeaturesType) => olStyle.Style;
    } & Record<string, any>;
    export type AddInteractionDrawOpts = {
      interactionKey: string;
      style?: olStyle.Style;
      drawType?: 'Polygon' | 'LineString' | 'MultiPolygon' | 'MultiLineString' | 'LinearRing' | 'Point' | 'MultiPoint' | 'GeometryCollection' | 'Circle';
      clickTolerance?: number;// 6px 绘制地图当按下鼠标和抬起鼠标左键之间的这段时间段内没超过多少px，默认添加一个点。相反如果超过了多少px，那么不会添加点，只是平移一下地图
      features?: any;
      snapTolerance?: number;// 12px 最后一点和第一点，吸附重合距离，小于12px,吸附到第一个点位
      maxPoints?: number;// 表示绘制单个要素（面和线）最多的点数限制，默认没有限制。
      minPoints?: number;// 表示绘制单个要素（面和线）需要的最少点数，面默认为 3，线默认为 2
      geometryFunction?: (coordinates: any, geometry: OLMAP.GeometryType) => OLMAP.GeometryType
      geometryName?: string;// 绘制的 geometry 的名称
      condition?: OLMAP.UseEventType.ConditionType;//规定了什么情况下触发 draw 操作，默认不需要特殊条件进行触发。一个函数，接受一个ol.MapBrowserEvent类型的参数，返回一个布尔值表示是否应该调用事件处理函数。默认情况下，增加一个顶点，类型为 ol.events.ConditionType
      eventList?: ({
        eventType?: "change" | "error" | "propertychange" | "change:active" | "drawstart" | "drawend" | "drawabort";
        eventCallback?: (e: any) => any
      })[];
      // 在修改或绘制矢量要素时处理矢量要素的捕捉 只要之前添加即可
      snapOpts?: {
        isAdd?: boolean;
        features?: any;
        edge?: boolean;// 默认为 true 与边缘对齐。
        vertex?: boolean;// 默认为 true 与顶点对齐
        pixelTolerance?: number; // 默认为 10
        source?: any;
        eventList?: ({
          eventType?: "change" | "error" | "propertychange" | "change:active";
          eventCallback?: (e: any) => any
        })[];
      } & Record<string, any>;
      modifyOpts?: {
        isAdd?: boolean;
        condition?: any;// 一个函数 指示是否将该事件视为向草绘添加顶点或移动 默认值为ol/events/condition.primaryAction.
        deleteCondition?: any;// 一个函数 指示是否应处理该事件 默认情况下，module:ol/events/condition.singleClick跟module:ol/events/condition.altKeyOnly导致顶点删除。
        insertVertexCondition?: any;// 一个函数，它接受module:ol/MapBrowserEvent~MapBrowserEvent并返回一个布尔值，以指示是否应将新折点添加到草绘要素。默认值为module:ol/events/condition.always.
        pixelTolerance?: number;// 10 像素容差，用于考虑指针足够接近线段或顶点以进行编辑
        style?: olStyle.Style;
        source?: any;
        hitDetection?: boolean;//配置后，将考虑根据其视觉外观修改点要素，而不是位于指针位置内。当pixelTolerancemodule:ol/layer/BaseVector~BaseVectorLayer，则仅考虑该图层上要素的渲染制图表达。
        features?: any;
        wrapX?: boolean;//false 在草图叠加层上水平环绕世界
        snapToPointer?: boolean;// 默认！hitDetection 在 中单击时，正在修改的顶点、点或线段会捕捉到指针坐标。
        eventList?: ({
          eventType?: "change" | "error" | "propertychange" | "change:active" | "modifystart" | "modifyend";
          eventCallback?: (e: any) => any
        })[];
      } & Record<string, any>;
      source?: any;
    } & Record<string, any>;
  }
  // --------------
  // 切片类型
  export type TileType<T extends TileSource> = layer.Tile<T>;
  // source类型
  export type SourceAllType = source.WMTS | source.XYZ | source.TileArcGISRest;
  // 区域，边界图层类型
  export type areaMapLayerType = layer.Vector<source.Vector<geom.Geometry | geom.Circle | geom.GeometryCollection | geom.Polygon & Record<string, any>>>;
  // 点位图层类型
  export type iconFeatureLayerType = layer.Vector<source.Vector<geom.Geometry | geom.Circle | geom.GeometryCollection | geom.Polygon | geom.Point>>;
  // 聚合图层类型
  export type clusterFeatureLayerType = layer.Vector<source.Cluster>;
  // 浮动层类型
  export type OverlayLayerType = Overlay;
  // 轨迹图层
  export type trajectoryLayerType = layer.Vector<source.Vector<geom.Geometry | geom.Circle | geom.GeometryCollection | geom.Polygon & Record<string, any>>>;
  // 勾绘的 geometry 类型
  export type GeometryType = geom.Polygon | geom.MultiPolygon | geom.LineString | geom.Point | geom.MultiPoint | geom.GeometryCollection | geom.LinearRing;

  // 数据源参数
  export namespace SourceType {
    export type ArcGisRest = {
      params?: { [key: string]: any };//ArcGIS Rest 参数
      hidpi?: boolean; // 默认true  //从远程服务器请求映像时使用该值
      crossOrigin?: string | undefined;// 常用：anonymous 加载图像的属性。请注意，如果要使用 Canvas 渲染器访问像素数据，则必须提供一个值。看crossOrigincrossOriginhttps://developer.mozilla.org/en-US/docs/Web/HTML/CORS_enabled_image了解更多详情。
      wrapX?: boolean;// 默认true 是否横向包装世界。
    } & Record<string, any> & TileImageBase;
    export type WMTS = {
      layer?: string;
      matrixSet?: string;
      format?: string;
      projection?: any;// 坐标系
      tileGrid?: any;
      style?: string;
      wrapX?: boolean;
    } & source.WMTS & TileImageBase;
    export type XYZ = {

    } & source.XYZ & TileImageBase;
    export type TileImageBase = {
      attributions?: any;
      attributionsCollapsible?: boolean;// (defaults to true) 属性是可折叠的
      cacheSize?: number;// 初始切片缓存大小。将自动增长以至少容纳视区中的切片数
      crossOrigin?: string | undefined;// 常用：anonymous 加载图像的属性。请注意，如果要使用 Canvas 渲染器访问像素数据，则必须提供一个值。看crossOrigincrossOriginhttps://developer.mozilla.org/en-US/docs/Web/HTML/CORS_enabled_image了解更多详情。
      imageSmoothing?: boolean; // 默认true  废弃 使用interpolate
      interpolate?: boolean; // 默认true 重新取样时使用插值。默认情况下，重新采样时使用线性插值。设置为 false 以改用最近的邻居。
      opaque?: boolean;// default false 图层是否不透明
      // params?: { [key: string]: any };//ArcGIS Rest 参数
      // hidpi?: boolean; // 默认true  //从远程服务器请求映像时使用该值
      projection?: any;//投影。默认值为视图投影。投影代码必须包含以 分隔的数字结束部分，或者整个代码必须形成有效的 ArcGIS 空间参考定义。
      reprojectionErrorThreshold?: number;// 默认0.5  //允许的最大重新投影误差（以像素为单位）。较高的值可以提高重新投影性能，但会降低精度
      state?: any;// 源状态
      tileClass?: any;//	用于实例化图像磁贴的类。默认值为module:ol/ImageTile~ImageTile.
      tileGrid?: any;// 磁贴网格。基于服务器支持的分辨率、切片大小和范围。如果未定义，则将使用默认格网：如果存在投影范围，则格网将基于该范围;否则，将使用基于原点为 0，0 的全局范围的格网。
      tileLoadFunction?: any;// 加载给定 URL 的磁贴的可选函数
      tilePixelRatio?: number;//	数字（默认为 1）	切片服务使用的像素比率。例如，如果磁贴服务将 256px x 256px 的磁贴进行广告宣传，但实际上发送了 512px x 512px 的图像（对于视网膜/hidpi 设备），则应将其设置为 。tilePixelRatio:2
      tileUrlFunction?: any;//	模块：ol/Tile~UrlFunction|定义	在给定图块坐标和投影的情况下获取图块 URL 的可选函数。
      url?: string;//地图服务或影像服务的 ArcGIS Rest 服务 URL。url 应包含 /MapServer 或 /ImageServer。
      wrapX?: boolean;// 默认true 是否横向包装世界。
      transition?: number;// 用于渲染的不透明度过渡的持续时间。要禁用不透明度过渡，请传递 。transition:0
      urls?: string[];// ArcGIS Rest service urls.当 ArcGIS 服务支持用于导出请求的多个 URL 时，请使用此选项
      key?: string;//	字符串|定义	用于正确提取缓存的可选切片键
      zDirection?: number;// 选择在整数缩放级别之间是使用具有较高或较低缩放级别的磁贴
    }
  }
}

// 轨迹数据
declare type TrajData = {
  lat?: number;
  lng?: number;
} & Record<string, any>;

// 点位数据
declare type Pointer = {
  lng?: number;
  lat?: number;
} & Record<string, any>;

/** 区域对象 */
declare type Area = {
  lng?: number;
  lat?: number;
  wkt?: string;
} & Omit<OLMAP.AreaFeatureType.AddAreaOpts, 'groupId' | 'textParam' | 'styleFn' | 'areaList'> & Record<string, any>;

declare type Icon = {
  lng?: number;
  lat?: number;
  iconSrc?: string;
} & Omit<OLMAP.IconFeatureType.AddIconOpts, 'iconSrc' | 'groupId' | 'textParam' | 'styleFn' | 'iconList' | 'defaultVisible'> & Record<string, any>;

// 图标参数
export declare type IconOpts = {
  /**
   * imgsource
   */
  image?: ImageStyle;
  /**
   * 偏移量
   * @default [0.5,30]
   */
  iconAnchor?: [number, number];
  /**
   * 原点位置，取值为bottom-left, bottom-right, top-left 或 top-right
   * @default top-right
   */
  iconAnchorOrigin?: 'bottom-left' | 'bottom-right' | 'top-left' | 'top-right';
  /**
   * 透明度
   * @default 1
   */
  iconOpacity?: number;
  /**
   * 缩放
   * @default 0.4
   */
  iconScale?: number;
};

// 文字参数
export declare type TextOpts = {
  /**
   * 文字可见性
   * @default true
   */
  textVisible?: boolean;
  /**
   * 文本显示字段
   * @default name
   */
  textParam?: string;
  /**
   * 文字偏移量
   * @default [20,0]
   */
  textOffset?: [number, number];
  /**
   * 取值“ left”，“ right”，“ center”，“ end”或“ start”。
   * @default left
   */
  textAlign?: 'left' | 'right' | 'center' | 'end' | 'start';
  /**
   * 文本的基准线 取值范围: ‘bottom’, ‘top’, ‘middle’, ‘alphabetic’, ‘hanging’, ‘ideographic’.
   * @default middle
   */
  textBaseline?: 'bottom' | 'top' | 'middle' | 'alphabetic' | 'hanging' | 'ideographic';
  /**
   * 字体样式
   * @default '12px PingFang'
   */
  font?: string;
  /**
   * 文本颜色
   * @default #fff
   */
  textColor?: string;
  /**
   * 文本背景颜色
   * @default rgba(0,0,0,.5)
   */
  textBackgroundColor?: string;
  /**
   * 内边距
   * @default [3,8,3,8]
   */
  textPadding?: [number, number, number, number];
};

export declare type Keys<T> = T extends object ? {
  [K in keyof T]-?: K;
}[keyof T] : never;

export declare type TOOLSTYPE = {
  FullScreen: FullScreen;
  Attribution: Attribution;
  MousePosition: MousePosition;
  OverviewMap: OverviewMap;
  Rotate: Rotate;
  ScaleLine: ScaleLine;
  Zoom: Zoom;
  ZoomSlider: ZoomSlider;
  ZoomToExtent: ZoomToExtent;
};

const OlMyMap = (
  {
    center,
    mapId,
    zoom = 10.6,
    minZoom,
    maxZoom,
    defaultLayersVisible = true,
    otherMapOpts,
    xScale = 1,
    yScale = 1,
    children,
  }: OlMyMapProps,
  ref: React.Ref<OlMyMapRef>
) => {
  const mapRef = useRef<ol.Map>();
  const mapBaseLayerRef = useRef<
    Map<string, OLMAP.TileType<OLMAP.SourceAllType>>
  >(new Map());
  const areaMapLayerRef = useRef<Map<string, OLMAP.areaMapLayerType>>(
    new Map()
  );
  const iconFeatureLayerRef = useRef<Map<string, OLMAP.iconFeatureLayerType>>(
    new Map()
  );
  const clusterFeatureLayerRef = useRef<
    Map<string, OLMAP.clusterFeatureLayerType>
  >(new Map());
  const positionMap = useRef<Map<string, number[] | undefined>>(new Map());
  const changeterMap = useRef<Map<string, any>>(new Map());
  const trajectoryLayerMap = useRef<Map<string, OLMAP.trajectoryLayerType>>(
    new Map()
  );
  const trajectoryOptsMap = useRef<Map<string, any>>(new Map());
  const listenerRef = useRef<
    Record<
      OLMAP.UseEventType.EventType,
      | Record<string, (e: OLMAP.UseEventType.MapBrowserEventType) => void>
      | undefined
    >
  >({
    change: undefined,
    click: undefined,
    singleclick: undefined,
    dblclick: undefined,
    moveend: undefined,
    movestart: undefined,
    pointerdrag: undefined,
    pointermove: undefined, //指针移动时触发
  });
  // 地图交互---点位选中
  const interactionSelectMap = useRef<Map<string, any>>(new Map());
  // 地图绘制图形
  const interactionDrawLayerMap = useRef<
    Map<string, layer.Vector<source.Vector<geom.Geometry>>>
  >(new Map());
  const interactionModifyMap = useRef<Map<string, any>>(new Map());
  const interactionSnapMap = useRef<Map<string, any>>(new Map());

  const initMap = () => {
    mapRef.current = new ol.Map({
      target: mapId || 'map',
      layers: [],
      view: new ol.View({
        center: center || [119.98258456970217, 30.302112473652276],
        zoom: zoom || 16,
        projection: 'EPSG:4326',
        minZoom: minZoom || 8,
        maxZoom: maxZoom || 18,
      }),
      ...otherMapOpts,
    });
    // 清除所有控件
    const ctls = mapRef.current.getControls();
    const ctlsLength = ctls.getArray().length;
    for (let i = 0; i < ctlsLength; i++) {
      mapRef.current.removeControl(ctls.getArray()[0]);
    }
    if (defaultLayersVisible) {
      setTDTLayer({
        projection: 'EPSG:3857',
        minZoom: minZoom || 8,
        maxZoom: maxZoom || 20,
        zIndex: 4,
        type: 'vec_w',
        greyMask: true,
        tk: '97aba9f6dc9933630adbf89af37d0052',
      });
      setTDTLWLayer({
        projection: 'EPSG:3857',
        minZoom: minZoom || 8,
        maxZoom: maxZoom || 20,
        zIndex: 4,
        type: 'cva_w',
        tk: '97aba9f6dc9933630adbf89af37d0052',
      });
    }
  };

  useEffect(() => {
    initMap();
  }, []);

  // 图层操作 --------------------
  const removeLayer = (key: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    let status = true;
    const removeItemLayer = (id: string) => {
      let status1 = true;
      let status2 = true;
      let status3 = true;
      let status4 = true;
      let status5 = true;
      let status6 = true;
      let status7 = true;
      // 底图图层
      if (isTileLayerVisible(id)) {
        status1 = removeBaseLayer(id);
      }
      // 边界图层
      else if (isAreaLayerVisible(id)) {
        status2 = areaFeaturesRemove(id);
      }
      // 图标图层
      else if (isIconLayerVisible(id)) {
        status3 = iconFeaturesRemove(id);
      }
      // 聚合点位图层
      else if (isClusterLayerVisible(id)) {
        status4 = clusterFeatureRemove(id);
      }
      // 覆盖层图层
      else if (isOverlayerLayerVisible(id)) {
        status5 = overlayerRemove(id);
      }
      // 覆盖层定时轮播图层
      else if (isOverlayerChangeLayerVisible(id)) {
        status6 = overlayChangeRemove(id);
      }
      // 轨迹图层
      else if (isTrajectoryLayerVisible(id)) {
        status7 = TrajectoryRemove(id);
      } else {
        return false;
      }
      return (
        status1 &&
        status2 &&
        status3 &&
        status4 &&
        status5 &&
        status6 &&
        status7
      );
    };
    if (Array.isArray(key)) {
      key?.forEach(id => {
        status = removeItemLayer(id);
      });
      return status;
    }
    return removeItemLayer(key);
  };

  // 图层显示或隐藏
  const setLayerVisible = (key: string | string[], visible: boolean) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    let status = true;
    const setItemLayerVisible = (id: string) => {
      let status1 = true;
      let status2 = true;
      let status3 = true;
      let status4 = true;
      let status5 = true;
      let status6 = true;
      let status7 = true;
      // 底图图层
      if (isTileLayerVisible(id)) {
        status1 = setTileLayerVisible(id, visible);
      }
      // 边界图层
      else if (isAreaLayerVisible(id)) {
        status2 = setAreaFeaturesVisible(id, visible);
      }
      // 图标图层
      else if (isIconLayerVisible(id)) {
        status3 = iconFeaturesSetVisible(id, visible);
      }
      // 聚合点位图层
      else if (isClusterLayerVisible(id)) {
        status4 = clusterFeatureSetVisible(id, visible);
      }
      // 覆盖层图层
      else if (isOverlayerLayerVisible(id)) {
        status5 = overlayerSetVisible(id, visible);
      }
      // 覆盖层定时轮播图层
      else if (isOverlayerChangeLayerVisible(id)) {
        status6 = overlayChangeSetVisible(id, visible);
      }
      // 轨迹图层
      else if (isTrajectoryLayerVisible(id)) {
        status7 = TrajectorySetVisible(id, visible);
      } else {
        return false;
      }
      return (
        status1 &&
        status2 &&
        status3 &&
        status4 &&
        status5 &&
        status6 &&
        status7
      );
    };
    if (Array.isArray(key)) {
      key?.forEach(id => {
        status = setItemLayerVisible(id);
      });
      return status;
    }
    return setItemLayerVisible(key);
  };

  const setLayerToggetVisible = (key: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    let status = true;
    const setItemLayerTogget = (id: string) => {
      let status1 = true;
      let status2 = true;
      let status3 = true;
      let status4 = true;
      let status5 = true;
      let status6 = true;
      let status7 = true;
      // 底图图层
      if (isTileLayerVisible(id)) {
        status1 = setTileLayerToggetVisible(id);
      }
      // 边界图层
      else if (isAreaLayerVisible(id)) {
        status2 = setAreaFeaturesToggetVisible(id);
      }
      // 图标图层
      else if (isIconLayerVisible(id)) {
        status3 = iconFeaturesSetToggetVisible(id);
      }
      // 聚合点位图层
      else if (isClusterLayerVisible(id)) {
        status4 = clusterFeatureSetToggetVisible(id);
      }
      // 覆盖层图层
      else if (isOverlayerLayerVisible(id)) {
        status5 = overlayerSetToggetVisible(id);
      }
      // 覆盖层定时轮播图层
      else if (isOverlayerChangeLayerVisible(id)) {
        status6 = overlayChangeSetToggetVisible(id);
      }
      // 轨迹图层
      else if (isTrajectoryLayerVisible(id)) {
        status7 = TrajectorySetToggetVisible(id);
      } else {
        return false;
      }
      return (
        status1 &&
        status2 &&
        status3 &&
        status4 &&
        status5 &&
        status6 &&
        status7
      );
    };
    if (Array.isArray(key)) {
      key?.forEach(id => {
        status = setItemLayerTogget(id);
      });
      return status;
    }
    return setItemLayerTogget(key);
  };

  // ----------------------

  // 视图view操作方法 -----------------------------------
  const setCenter = (coordinate: [number, number]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!coordinate?.[0] || !coordinate?.[1]) return false;
    mapRef?.current?.getView()?.setCenter(coordinate);
    return true;
  };

  const setZoom = (zoom: number) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!zoom) return false;
    mapRef?.current?.getView()?.setZoom(zoom);
    return true;
  };

  const setFit = (str: string, padding: [number, number, number, number]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const geoms = readGeometry(str);
    if (!geoms || !padding) return false;
    mapRef?.current?.getView()?.fit(geoms as any, {
      size: mapRef?.current?.getSize(),
      padding: padding,
    });
    return true;
  };

  const getCenter = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return undefined;
    }
    return mapRef?.current?.getView()?.getCenter();
  };

  const getZoom = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return undefined;
    }
    return mapRef?.current?.getView()?.getZoom();
  };

  // -----------------------------
  // 地图底图加载------------------------------------------------
  // 设置天地图map
  const setTDTLayer = (
    options: OLMAP.LayerType.TileLayerOpts & OLMAP.LayerType.TiandituOpts
  ) => {
    const { domain, tk, greyMask, type, ...otherOptions } = options;
    const epsg = type?.split('_')?.[1] === 'c' ? 'EPSG:4326' : 'EPSG:3857';
    const projection = proj.get(epsg)!;
    const projectionExtent = projection.getExtent();
    const size = extent.getWidth(projectionExtent) / 256;
    const length = 19;
    const resolutions = new Array<number>(length);
    const matrixIds = new Array<string>(length);
    // const matrixIds = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14'];
    for (let i = 0; i < length; i += 1) {
      const pow = Math.pow(2, i);
      resolutions[i] = size / pow;
      matrixIds[i] = `${i}`;
    }

    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const key = `ol_tianditu`;
    if (mapBaseLayerRef?.current?.get(key)) {
      return false;
    }
    const baseUrl = `http://${domain ||
      't0'}.tianditu.gov.cn/${type}/wmts?tk=${tk ||
      '97aba9f6dc9933630adbf89af37d0052'}`;
    const layerTdt = new layer.Tile({
      source: new source.WMTS({
        url: baseUrl,
        layer: type?.split('_')?.[0],
        style: 'default',
        matrixSet: type?.split('_')?.[1],
        format: 'tiles',
        version: '1.0.0',
        wrapX: true,
        projection: projection,
        tileGrid: new tilegridWMTS({
          origin: extent.getTopLeft(projectionExtent),
          resolutions,
          matrixIds,
        }),
        tileLoadFunction: greyMask
          ? (imageTitle: any, src: any) => {
              const img = new Image();
              img.crossOrigin = '';
              img.onload = () => {
                const canvas = document.createElement('canvas');
                const w = img.width;
                const h = img.height;
                canvas.width = w;
                canvas.height = h;
                const context: any = canvas.getContext('2d');
                context.drawImage(img, 0, 0, w, h, 0, 0, w, h);
                const imgData = context.getImageData(0, 0, w, h);
                for (let i = 0; i < imgData.height; i++) {
                  for (let j = 0; j < imgData.width; j++) {
                    const x = i * 4 * imgData.width + j * 4;
                    // canvas置灰
                    const r = imgData.data[x];
                    const g = imgData.data[x + 1];
                    const b = imgData.data[x + 2];
                    const avg = (r + g + b) / 2.8;
                    // // 3.1
                    // // 注意这里：avg 是灰度值，如果只需要灰度那就直接赋值，需要倒置（invert）的，用 255 减去灰度值
                    imgData.data[x] = imgData.data[x + 1] = imgData.data[
                      x + 2
                    ] = avg;
                  }
                }
                context.putImageData(imgData, 0, 0);
                imageTitle.getImage().src = canvas.toDataURL('image/png');
              };
              img.src = src;
            }
          : undefined,
      }),
      zIndex: 4,
      ...otherOptions,
    });
    mapBaseLayerRef?.current?.set(key, layerTdt);
    mapRef?.current?.addLayer(layerTdt);
    return true;
  };

  // 设置天地图路网
  const setTDTLWLayer = (
    options: OLMAP.LayerType.TileLayerOpts &
      Omit<OLMAP.LayerType.TiandituOpts, 'type'>
  ) => {
    const { domain, tk, greyMask, type, ...otherOptions } = options;
    const epsg = type?.split('_')?.[1] === 'c' ? 'EPSG:4326' : 'EPSG:3857';
    const projection = proj.get(epsg)!;
    const projectionExtent = projection.getExtent();
    const size = extent.getWidth(projectionExtent) / 256;
    const length = 19;
    const resolutions = new Array<number>(length);
    const matrixIds = new Array<string>(length);
    // const matrixIds = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14'];
    for (let i = 0; i < length; i += 1) {
      const pow = Math.pow(2, i);
      resolutions[i] = size / pow;
      matrixIds[i] = `${i}`;
    }

    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const key = `ol_tianditu_lw`;
    if (mapBaseLayerRef?.current?.get(key)) {
      return false;
    }
    const baseUrl = `http://${domain || 't0'}.tianditu.gov.cn/${type ||
      'cva_w'}/wmts?tk=${tk || '97aba9f6dc9933630adbf89af37d0052'}`;
    const layerTdt = new layer.Tile({
      source: new source.WMTS({
        url: baseUrl,
        layer: type?.split('_')?.[0] || 'cva',
        style: 'default',
        matrixSet: type?.split('_')?.[1] || 'w',
        format: 'tiles',
        version: '1.0.0',
        wrapX: true,
        projection: projection,
        tileGrid: new tilegridWMTS({
          origin: extent.getTopLeft(projectionExtent),
          resolutions,
          matrixIds,
        }),
        tileLoadFunction: greyMask
          ? (imageTitle: any, src: any) => {
              const img = new Image();
              img.crossOrigin = '';
              img.onload = () => {
                const canvas = document.createElement('canvas');
                const w = img.width;
                const h = img.height;
                canvas.width = w;
                canvas.height = h;
                const context: any = canvas.getContext('2d');
                context.drawImage(img, 0, 0, w, h, 0, 0, w, h);
                const imgData = context.getImageData(0, 0, w, h);
                for (let i = 0; i < imgData.height; i++) {
                  for (let j = 0; j < imgData.width; j++) {
                    const x = i * 4 * imgData.width + j * 4;
                    // canvas置灰
                    const r = imgData.data[x];
                    const g = imgData.data[x + 1];
                    const b = imgData.data[x + 2];
                    const avg = (r + g + b) / 2.8;
                    // // 3.1
                    // // 注意这里：avg 是灰度值，如果只需要灰度那就直接赋值，需要倒置（invert）的，用 255 减去灰度值
                    imgData.data[x] = imgData.data[x + 1] = imgData.data[
                      x + 2
                    ] = avg;
                  }
                }
                context.putImageData(imgData, 0, 0);
                imageTitle.getImage().src = canvas.toDataURL('image/png');
              };
              img.src = src;
            }
          : undefined,
      }),
      zIndex: 5,
      ...otherOptions,
    });
    mapBaseLayerRef?.current?.set(key, layerTdt);
    mapRef?.current?.addLayer(layerTdt);
    return true;
  };

  // 设置百度地图
  const setBaiduLayer = (options: OLMAP.LayerType.TileLayerOpts) => {
    const { customid, ...otherOptions } = options;
    const baiduMercatorProj = new proj.Projection({
      code: 'baidu',
      extent: extent.applyTransform(
        [72.004, 0.8293, 137.8347, 55.8271],
        projzh.ll2bmerc
      ),
      units: 'm',
    });
    proj.addProjection(baiduMercatorProj);
    proj.addCoordinateTransforms(
      'EPSG:4326',
      baiduMercatorProj,
      projzh.ll2bmerc,
      projzh.bmerc2ll
    );
    proj.addCoordinateTransforms(
      'EPSG:3857',
      baiduMercatorProj,
      projzh.smerc2bmerc,
      projzh.bmerc2smerc
    );
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const key = 'ol_baidu';
    if (mapBaseLayerRef?.current?.get(key)) {
      return false;
    }
    const baiduLayer = new layer.Tile({
      source: new source.XYZ({
        crossOrigin: 'anonymous',
        // projection: ol.proj.get("EPSG:3857"),
        projection: 'baidu',
        tileGrid: new TileGrid({
          origin: [0, 0],
          resolutions: bmercResolutions,
          tileSize: [256, 256],
          extent: extent.applyTransform(
            [72.004, 0.8293, 137.8347, 55.8271],
            projzh.ll2bmerc
          ),
        }),
        tileUrlFunction: (tileCoord: any[]) => {
          if (!tileCoord) {
            return '';
          }
          let [z, x, y] = tileCoord;
          // if (x < 0) {
          //   x = "M" + (-x);
          // }
          // if (y < 0) {
          //   y = "M" + (-y);
          // }
          // return `http://online0.map.bdimg.com/onlinelabel/?qt=tile&x=${x}&y=${-y - 1}&z=${z}&styles=pl&udt=20210506&scaler=1&p=1`
          return (
            'https://api.map.baidu.com/customimage/tile?&x=' +
            x +
            '&y=' +
            (-y - 1) +
            '&z=' +
            z +
            '&udt=20210506&scale=2&ak=8d6c8b8f3749aed6b1aff3aad6f40e37&customid=' +
            (customid || 'grayscale')
          );
        },
        // tileUrlFunction: (tileCoord: any[]) => {
        //   if (!tileCoord) {
        //     return "";
        //   }
        //   let [z, x, y] = tileCoord;
        //   if (x < 0) {
        //     x = "M" + (-x);
        //   }
        //   if (y < 0) {
        //     y = "M" + (-y);
        //   }
        //   return `http://online0.map.bdimg.com/onlinelabel/?qt=tile&x=${x}&y=${y}&z=${z}&styles=pl&udt=20210506&scaler=1&p=1`
        // }
      }),
      ...otherOptions,
    });
    mapBaseLayerRef?.current?.set(key, baiduLayer);
    mapRef?.current?.addLayer(baiduLayer);
    // setCenter([120.0434120129589, 30.27750873257137]);
    return true;
  };

  // 设置高德地图
  const setGaoDeLayer = (
    options: OLMAP.LayerType.TileLayerOpts & OLMAP.LayerType.GaodeLayerOpts
  ) => {
    const { type = '矢量地图带注记', ...otherOptions } = options;
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const key = 'ol_gaode';
    if (mapBaseLayerRef?.current?.get(key)) {
      return false;
    }
    let url =
      'https://webrd0{1-4}.is.autonavi.com/appmaptile?lang=zh_cn&size=1&scale=1&style=8&x={x}&y={y}&z={z}';
    switch (type) {
      case '道路带注记':
        url =
          'http://webst0{1-4}.is.autonavi.com/appmaptile?x={x}&y={y}&z={z}&lang=zh_cn&size=1&scale=1&style=8';
        break;
      case '道路不带注记':
        url =
          'http://wprd0{1-4}.is.autonavi.com/appmaptile?x={x}&y={y}&z={z}&lang=zh_cn&size=1&scl=1&style=8&ltype=11';
        break;
      case '矢量带标注':
        url =
          'http://wprd0{1-4}.is.autonavi.com/appmaptile?lang=zh_cn&size=1&style=7&x={x}&y={y}&z={z}';
        break;
      case '影像不带注记':
        url =
          'http://webst0{1-4}.is.autonavi.com/appmaptile?style=6&x={x}&y={y}&z={z}';
        break;
      case '矢量地图带注记':
        url =
          'https://webrd0{1-4}.is.autonavi.com/appmaptile?lang=zh_cn&size=1&scale=1&style=8&x={x}&y={y}&z={z}';
        break;
      case '矢量地图不带注记':
        url =
          'http://wprd0{1-4}.is.autonavi.com/appmaptile?lang=zh_cn&size=1&style=7&x={x}&y={y}&z={z}&scl=1&ltype=3';
        break;
      default:
        url =
          'https://webrd0{1-4}.is.autonavi.com/appmaptile?lang=zh_cn&size=1&scale=1&style=8&x={x}&y={y}&z={z}';
        break;
    }
    const gaodeLayer = new layer.Tile({
      source: new source.XYZ({
        projection: 'EPSG:3857',
        maxZoom: 20,
        url,
      }),
      ...otherOptions,
    });
    mapBaseLayerRef?.current?.set(key, gaodeLayer);
    mapRef?.current?.addLayer(gaodeLayer);
    return true;
  };
  /**
   * 添加影像切片图层
   * @param options TileLayer配置项
   */
  const setYXLayer = (
    options: OLMAP.LayerType.TileLayerOpts & {
      access_token?: string;
      curl?: string;
      sku?: string;
    }
  ) => {
    const {
      access_token = 'pk.eyJ1IjoieHp4dGVzdCIsImEiOiJjbDUwcm5lbHczYjVkM2NscDIwZ2tmM3o2In0.2cNDZDLoc2xu25O8wjAmYA',
      sku = '101ay5ZW6LQf7',
      curl,
      ...otherOptions
    } = options;
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const key = 'ol_yingxiang';
    if (mapBaseLayerRef?.current?.get(key)) {
      return false;
    }
    const yxTileLayer = new layer.Tile(
      Object.assign(
        {},
        Object.assign(
          {},
          {
            visible: true,
            zIndex: 4,
            source: new source.XYZ({
              url:
                curl ||
                `https://api.mapbox.com/v4/mapbox.satellite/{z}/{x}/{y}.webp?sku=${sku}&access_token=${access_token}`,
            }),
          },
          otherOptions
        ),
        { className: key }
      )
    );
    mapRef?.current?.addLayer(yxTileLayer);
    mapBaseLayerRef?.current?.set(key, yxTileLayer);
    return true;
  };

  // 设置自定义底图图层
  const setUpYouLayer = (options: OLMAP.LayerType.TileUpYouLayerOpts) => {
    const { key, ...otherOpts } = options;
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (mapBaseLayerRef?.current?.get(key)) {
      return false;
    }
    const upYouLayer = new layer.Tile(otherOpts);
    mapRef?.current?.addLayer(upYouLayer);
    mapBaseLayerRef?.current?.set(key, upYouLayer);
    return true;
  };

  // 设置wmts底图
  const setWmtsLayer = (options: OLMAP.LayerType.TileWmtsLayerOpts) => {
    const { url, key, wmtsSourceOpts, ...layerOpts } = options;
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (mapBaseLayerRef?.current?.get(key)) {
      return false;
    }

    const projection: any = proj.get('EPSG:4326');
    const projectionExtent = projection.getExtent();
    const size = extent.getWidth(projectionExtent) / 256;
    const resolutions: any[] = [];
    const matrixIds: any[] = [];
    for (let z = 2; z < 19; z += 1) {
      resolutions[z] = size / 2 ** z;
      matrixIds[z] = z;
    }
    const wmtsSource = new source.WMTS({
      url:
        url ||
        'https://yt.zjasm.net/apigw/services/tilelayer4490/ygyx2021/latest/68cb09c2-31e1-4d45-addd-3dd4c9c3a0f3',
      layer: 'imgmap',
      matrixSet: 'esritilematirx',
      format: 'image/jpgpng',
      projection: projection,
      tileGrid: new WMTSTileGrid({
        origin: extent.getTopLeft(projectionExtent),
        resolutions: resolutions,
        matrixIds: matrixIds,
      }),
      style: 'default',
      wrapX: true,
      ...wmtsSourceOpts,
    });
    const wmtsService = new layer.Tile({
      source: wmtsSource,
      zIndex: 1,
      ...layerOpts,
    });
    mapRef?.current?.addLayer(wmtsService);
    mapBaseLayerRef?.current?.set(key, wmtsService);
    return true;
  };

  // 设置wmts底图
  const setTileArcGISRestLayer = (
    options: OLMAP.LayerType.TileArcGISRestLayerOpts
  ) => {
    const { url, key, arcGisRestopts, ...layerOpts } = options;
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (mapBaseLayerRef?.current?.get(key)) {
      return false;
    }
    const wmtsSource = new source.TileArcGISRest({
      url:
        url ||
        'https://yt.zjasm.net/qzgt18/d889d009190e4b23a470d11f696d4c54/arcgis/rest/services/GCS330800G100101_JCDL/GCS330800G100101_JCDL_XZQH_XZ_2022/MapServer',
      wrapX: true,
      crossOrigin: 'anonymous',
      ...arcGisRestopts,
    });
    const wmtsService = new layer.Tile({
      source: wmtsSource,
      zIndex: 1,
      ...layerOpts,
    });

    mapRef?.current?.addLayer(wmtsService);
    mapBaseLayerRef?.current?.set(key, wmtsService);
    return true;
  };

  const setTileLayerVisible = (
    className: string | string[],
    visible: boolean
  ) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!className) return false;
    const changeItemVisible = (key: string, visible1: boolean) => {
      const layer = mapBaseLayerRef?.current?.get(key);
      if (layer) {
        layer.setVisible(visible1);
        return true;
      }
      return false;
    };
    let status = true;
    if (Array.isArray(className)) {
      className?.forEach((it: string) => {
        const status1 = changeItemVisible(it, visible);
        if (!status1) status = false;
      });

      return status;
    }
    const status1 = changeItemVisible(className, visible);
    return status1;
  };

  const setTileLayerToggetVisible = (className: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!className) return false;
    const changeItemVisible = (key: string) => {
      const layer = mapBaseLayerRef?.current?.get(key);
      if (layer) {
        layer.setVisible(!layer.getVisible());
        return true;
      }
      return false;
    };
    let status = true;
    if (Array.isArray(className)) {
      className?.forEach((it: string) => {
        const status1 = changeItemVisible(it);
        if (!status1) status = false;
      });

      return status;
    }
    const status1 = changeItemVisible(className);
    return status1;
  };

  const isTileLayerVisible = (className: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!className) return false;
    let status = true;
    if (Array.isArray(className)) {
      className?.forEach((it: string) => {
        const layer = mapBaseLayerRef?.current?.get(it);
        if (!layer) status = false;
      });

      return status;
    }
    const layer = mapBaseLayerRef?.current?.get(className);
    return !!layer;
  };

  const tileLayerClear = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const layers = Array.from(mapBaseLayerRef?.current?.values());
    layers?.forEach((itm: any) => {
      mapRef?.current?.removeLayer(itm);
    });
    mapBaseLayerRef?.current?.clear();
    return true;
  };

  const tileLayerHideAll = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const layers = Array.from(mapBaseLayerRef?.current?.values());
    layers?.forEach((itm: any) => {
      itm?.setVisible(false);
    });
    return true;
  };

  const tileLayerShowAll = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const layers = Array.from(mapBaseLayerRef?.current?.values());
    layers?.forEach((itm: any) => {
      itm?.setVisible(true);
    });
    return true;
  };

  // 清除图层 多个图层key用,分割开
  const removeBaseLayer = (key: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    let status = true;
    if (Array.isArray(key)) {
      key?.forEach(name => {
        const mapLayer = mapBaseLayerRef?.current?.get(name);
        if (mapLayer) {
          mapRef?.current?.removeLayer(mapLayer);
          mapBaseLayerRef?.current?.delete(name);
        } else {
          status = false;
        }
      });
    } else {
      const mapLayer = mapBaseLayerRef?.current?.get(key);
      if (mapLayer) {
        mapRef?.current?.removeLayer(mapLayer);
        mapBaseLayerRef?.current?.delete(key);
      } else {
        status = false;
      }
    }
    return status;
  };
  // ------------------------------------------------

  // 区域图层添加边界------------------------------
  /**新增边界 */
  const areaFeaturesAdd = (options: OLMAP.AreaFeatureType.AddAreaOpts) => {
    const {
      groupId: id,
      textParam = 'name',
      areaList,
      isLock,
      defaultVisible,
      zIndex,
      ...otherOpts
    } = options;
    const groupId = GetId.getAreaGroupId(id);
    const styleOpts = loadshGet.omit(otherOpts, [
      'zIndex',
      'groupId',
      'areaList',
      'textParam',
      'styleFn',
      'isLock',
      'defaultVisible',
    ]);
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!groupId || !areaList?.length) {
      return false;
    }
    if (!isLock && areaMapLayerRef?.current?.has(groupId)) {
      areaFeaturesRemove(id);
    }
    if (isLock && areaMapLayerRef?.current?.has(groupId)) {
      return false;
    }
    let features: any[] = [];
    areaList?.forEach((area: Area) => {
      const { wkt, styleFn, ...areaOpts } = area;
      const geometry = readGeometry(wkt!);
      if (geometry instanceof Polygon) {
        let featureItem: ol.Feature<geom.Geometry> &
          Record<string, any> = new Feature({ geometry: geometry });
        featureItem.userData = area;
        featureItem.groupId = groupId;
        let params = Object.assign({}, { ...areaOpts! }, { ...styleOpts! });
        // console.log('xxxx11', params, Object.assign({}, params, { text: area[textParam] }));
        featureItem.setStyle(
          !styleFn
            ? createAreaStyle(
                Object.assign({}, params, { text: area[textParam] })
              )
            : styleFn(area)
        );
        features?.push(featureItem);
      }

      if (geometry instanceof MultiPolygon) {
        let polyList = getpolyArea(geometry?.getPolygons());
        polyList.forEach((subItem: any, index: number) => {
          let featureItem: ol.Feature<geom.Geometry> &
            Record<string, any> = new Feature({
            geometry: subItem,
          });
          featureItem.userData = area;
          featureItem.groupId = groupId;
          let _a = Object.assign({}, { ...areaOpts! }, { ...styleOpts! }),
            _b = _a.textVisible,
            textVisible = _b === void 0 ? true : _b,
            opts = loadshGet.omit(_a, ['textVisible']);
          // console.log('xxxx', _a, opts);
          featureItem.setStyle(
            !styleFn
              ? createAreaStyle(
                  Object.assign({}, opts, {
                    text: area[textParam],
                    textVisible: index === 0 && textVisible,
                  })
                )
              : styleFn(area)
          );
          features?.push(featureItem);
        });
      }
    });
    if (!features.length) {
      return false;
    }
    const source = new VectorSource();
    source?.addFeatures(features);
    const areaLayer = new VectorLayer({
      source: source,
      visible: defaultVisible,
      zIndex: zIndex,
    });
    mapRef?.current?.addLayer(areaLayer);
    areaMapLayerRef?.current?.set(groupId, areaLayer);
    return features;
  };

  /**移除图层 */
  const areaFeaturesRemove = (groupId: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    let status = true;
    if (Array.isArray(groupId)) {
      const groupIds = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getAreaGroupId(it));
      groupIds?.forEach(it => {
        const layerArea = areaMapLayerRef?.current?.get(it);
        if (layerArea) {
          mapRef?.current?.removeLayer(layerArea);
          areaMapLayerRef?.current?.delete(it);
        } else {
          status = false;
        }
      });
    } else {
      const idGroup = GetId.getAreaGroupId(groupId);
      const layerArea = areaMapLayerRef?.current?.get(idGroup);
      if (layerArea) {
        mapRef?.current?.removeLayer(layerArea);
        areaMapLayerRef?.current?.delete(idGroup);
      } else {
        status = false;
      }
    }

    return status;
  };

  /**设置图层显隐 */
  const setAreaFeaturesVisible = (
    groupId: string | string[],
    visible: boolean
  ) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!groupId) return false;
    let status = true;
    if (Array.isArray(groupId)) {
      const keys = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getAreaGroupId(it));
      keys?.forEach((it: string) => {
        const areaMap = areaMapLayerRef?.current?.get(it);
        if (areaMap) {
          areaMap?.setVisible(visible);
        } else {
          status = false;
        }
      });
    } else {
      const key = GetId.getAreaGroupId(groupId);
      const areaMap = areaMapLayerRef?.current?.get(key);
      if (areaMap) {
        areaMap?.setVisible(visible);
      } else {
        status = false;
      }
    }

    return status;
  };

  const setAreaFeaturesToggetVisible = (groupId: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!groupId) return false;
    let status = true;
    if (Array.isArray(groupId)) {
      const keys = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getAreaGroupId(it));
      keys?.forEach((it: string) => {
        const areaMap = areaMapLayerRef?.current?.get(it);
        if (areaMap) {
          areaMap?.setVisible(!areaMap?.getVisible());
        } else {
          status = false;
        }
      });
    } else {
      const key = GetId.getAreaGroupId(groupId);
      const areaMap = areaMapLayerRef?.current?.get(key);
      if (areaMap) {
        areaMap?.setVisible(!areaMap?.getVisible());
      } else {
        status = false;
      }
    }

    return status;
  };

  const isAreaLayerVisible = (groupId: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!groupId) return false;
    let status = true;
    if (Array.isArray(groupId)) {
      const keys = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getAreaGroupId(it));
      keys?.forEach((it: string) => {
        const layer = areaMapLayerRef?.current?.get(it);
        if (!layer) status = false;
      });

      return status;
    }
    const key = GetId.getAreaGroupId(groupId);
    const layer = areaMapLayerRef?.current?.get(key);
    return !!layer;
  };

  /**清除边界图层 */
  const clearAreaFeatures = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const areaList = Array.from(areaMapLayerRef?.current?.values());
    areaList?.forEach((it: any) => {
      mapRef?.current?.removeLayer(it);
    });
    areaMapLayerRef?.current?.clear();
    return true;
  };

  /**样式配置 */
  const createAreaStyle = (
    feature: {
      text: string;
    } & Pick<
      OLMAP.AreaFeatureType.AddAreaOpts,
      | 'textVisible'
      | 'textParam'
      | 'textOffset'
      | 'textAlign'
      | 'textBaseline'
      | 'font'
      | 'textColor'
      | 'textBackgroundColor'
      | 'textPadding'
      | 'zIndex'
      | 'styleFn'
      | 'isLock'
      | 'defaultVisible'
      | 'backgroundColor'
      | 'borderSize'
      | 'borderColor'
    >
  ) => {
    let _a = Object.assign({}, { ...DEFAULTTEXT }, { ...feature! }),
      _b = _a.backgroundColor,
      backgroundColor = _b === void 0 ? 'rgba(137,119,251,0.5)' : _b,
      _c = _a.borderColor,
      borderColor = _c === void 0 ? '#fff' : _c,
      _d = _a.borderSize,
      borderSize = _d === void 0 ? 2 : _d,
      textVisible = _a.textVisible,
      text = _a.text,
      font = _a.font,
      textBaseline = _a.textBaseline,
      textAlign = _a.textAlign,
      textColor = _a.textColor,
      textBackgroundColor = _a.textBackgroundColor,
      textPadding = _a.textPadding;

    return new olStyle.Style({
      fill: new olStyle.Fill({
        color: backgroundColor,
      }),
      stroke: new olStyle.Stroke({
        color: borderColor,
        width: borderSize,
      }),
      text: textVisible
        ? new olStyle.Text({
            textAlign: textAlign,
            textBaseline: textBaseline,
            font: font,
            text: text,
            // 填充样式
            fill: new olStyle.Fill({
              color: textColor,
            }),
            backgroundFill: new olStyle.Fill({
              color: textBackgroundColor,
            }),
            padding: textPadding,
          })
        : undefined,
    });
  };

  const readGeometry = (str: string) => {
    if (!str) return null;
    let newstr;
    let wkt = new WKT();
    try {
      newstr = WKTTOOLS.convert(JSON.parse(str));
    } catch (error) {
      newstr = str;
    }
    return wkt.readGeometry(newstr);
  };
  // ---------------------------------------------

  // 点位图层 ------------------------------
  const iconFeaturesAdd = (opts: OLMAP.IconFeatureType.AddIconOpts) => {
    const {
      isLock = true,
      iconList,
      groupId: id,
      iconSrc: psrc,
      zIndex = 20,
      textParam = 'name',
      styleFn,
      defaultVisible = true,
    } = opts;
    const styleOpts = loadshGet.omit(opts, [
      'zIndex',
      'groupId',
      'iconList',
      'iconSrc',
      'textParam',
      'styleFn',
      'isLock',
      'defaultVisible',
    ]);
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const groupId = GetId.getIconGroupId(id);
    if (!groupId || !iconList?.length) return false;
    if (!isLock && iconFeatureLayerRef?.current?.has(groupId)) {
      iconFeaturesRemove(id);
    }
    if (isLock && iconFeatureLayerRef?.current?.has(groupId)) {
      return false;
    }
    const newList = filterLngLatByList(iconList)?.map((icon: any) => {
      const { lng, lat, iconSrc: csrc } = icon;
      const src =
        csrc || psrc;
      const iconOpts = loadshGet.omit(icon, ['lng', 'lat', 'iconSrc']);
      const feature: any = new Feature({
        geometry: new geom.Point([lng, lat]),
      });
      feature.userData = icon;
      feature.groupId = groupId;
      const params = getParams(iconOpts, styleOpts);
      feature.setStyle(
        !styleFn
          ? createIconStyle(
              Object.assign({}, Object.assign({}, { iconSrc: src }, params), {
                text: icon[textParam],
              })
            )
          : styleFn(icon)
      );
      return feature;
    });
    if (!newList.length) return false;
    const iconSource = new source.Vector();
    const iconLayer = new layer.Vector({
      source: iconSource,
      visible: defaultVisible,
      zIndex: zIndex,
    });
    iconSource?.addFeatures(newList);
    mapRef?.current?.addLayer(iconLayer);
    iconFeatureLayerRef?.current?.set(groupId, iconLayer);
    return newList;
  };

  /**传入未处理的groupId，即外部设置的id */
  const iconFeaturesRemove = (groupId: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    let status = true;
    if (Array.isArray(groupId)) {
      const groupIds = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getIconGroupId(it));
      groupIds?.forEach(it => {
        const layerArea = iconFeatureLayerRef?.current?.get(it);
        if (layerArea) {
          mapRef?.current?.removeLayer(layerArea);
          iconFeatureLayerRef?.current?.delete(it);
        } else {
          status = false;
        }
      });
    } else {
      const key = GetId.getIconGroupId(groupId);
      const layerArea = iconFeatureLayerRef?.current?.get(key);
      if (layerArea) {
        mapRef?.current?.removeLayer(layerArea);
        iconFeatureLayerRef?.current?.delete(key);
      } else {
        status = false;
      }
    }

    return status;
  };

  const iconFeaturesSetVisible = (
    groupId: string | string[],
    visible: boolean
  ) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!groupId) return false;
    let status = true;
    if (Array.isArray(groupId)) {
      const keys = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getIconGroupId(it));
      keys?.forEach((it: string) => {
        const areaMap = iconFeatureLayerRef?.current?.get(it);
        if (areaMap) {
          areaMap?.setVisible(visible);
        } else {
          status = false;
        }
      });
    } else {
      const key = GetId.getIconGroupId(groupId);
      const areaMap = iconFeatureLayerRef?.current?.get(key);
      if (areaMap) {
        areaMap?.setVisible(visible);
      } else {
        status = false;
      }
    }

    return status;
  };

  const iconFeaturesSetToggetVisible = (groupId: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!groupId) return false;
    let status = true;
    if (Array.isArray(groupId)) {
      const keys = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getIconGroupId(it));
      keys?.forEach((it: string) => {
        const areaMap = iconFeatureLayerRef?.current?.get(it);
        if (areaMap) {
          areaMap?.setVisible(!areaMap?.getVisible());
        } else {
          status = false;
        }
      });
    } else {
      const key = GetId.getIconGroupId(groupId);
      const areaMap = iconFeatureLayerRef?.current?.get(key);
      if (areaMap) {
        areaMap?.setVisible(!areaMap?.getVisible());
      } else {
        status = false;
      }
    }

    return status;
  };

  const isIconLayerVisible = (groupId: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!groupId) return false;
    let status = true;
    if (Array.isArray(groupId)) {
      const keys = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getIconGroupId(it));
      keys?.forEach((it: string) => {
        const layer = iconFeatureLayerRef?.current?.get(it);
        if (!layer) status = false;
      });

      return status;
    }
    const key = GetId.getIconGroupId(groupId);
    const layer = iconFeatureLayerRef?.current?.get(key);
    return !!layer;
  };

  const iconFeaturesClear = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const iconList = Array.from(iconFeatureLayerRef?.current?.values());
    iconList?.forEach((it: any) => {
      mapRef?.current?.removeLayer(it);
    });
    iconFeatureLayerRef?.current?.clear();
    return true;
  };

  // 修改位置----暂不完善
  const iconFeatureSetPosition = (
    groupId: string,
    find: (data: { userData: Icon }) => boolean,
    coordinate: number[]
  ) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const groupIdNew = GetId.getIconGroupId(groupId);
    const iconFeature = iconFeatureLayerRef?.current?.get(groupIdNew);
    const features = iconFeature?.getSource()?.getFeatures();
    if (!features) return false;
    const feature = features.find(find as any);
    const geometry = feature?.getGeometry() as geom.Point;
    return geometry?.setCoordinates(coordinate);
  };

  const iconFeatureFocus = (
    groupId: string | string[],
    padding?: [number, number, number, number]
  ) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const myPadding = padding || [100, 100, 100, 100];
    if (Array.isArray(groupId)) {
      const groupIds = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getIconGroupId(it));
      const mainList1 = groupIds?.map((id: string) => {
        const iconFeature = iconFeatureLayerRef?.current?.get(id) as any;
        const data = iconFeature?.userData;
        return data;
      });
      const wkt1 = setFocus(mainList1);
      return setFit(wkt1!, myPadding);
    }
    const groupIds = [groupId]
      ?.filter((it: any) => it)
      ?.map((it: string) => GetId.getIconGroupId(it));
    const mainList = groupIds?.map((id: string) => {
      const iconFeature = iconFeatureLayerRef?.current?.get(id) as any;
      const data = iconFeature?.userData;
      return data;
    });
    const wkt = setFocus(mainList);
    return setFit(wkt!, myPadding);
  };

  const iconFeatureCheck = (groupId: string) => {
    const key = GetId.getIconGroupId(groupId);
    const layer = iconFeatureLayerRef?.current?.get(key);
    return !!layer;
  };
  // ------------------------------

  // 聚合点位-----------------------
  const clusterFeatureAdd = (opts: OLMAP.ClusterFeatureType.AddClusterOpts) => {
    const {
      isLock = true,
      iconList,
      groupId: id,
      zIndex = 20,
      styleFn,
      defaultVisible = true,
      distance = 200,
    } = opts;
    const styleOpts: any = loadshGet.omit(opts, [
      'zIndex',
      'groupId',
      'iconList',
      'iconSrc',
      'textParam',
      'styleFn',
      'isLock',
      'defaultVisible',
    ]);
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const groupId = GetId.getClusterGroupId(id);
    if (!groupId) return false;
    if (!isLock && clusterFeatureLayerRef?.current?.has(groupId)) {
      clusterFeatureRemove(id);
    }
    if (isLock && clusterFeatureLayerRef?.current?.has(groupId)) {
      return false;
    }
    const newList = filterLngLatByList(iconList)?.map((icon: any) => {
      const { lng, lat } = icon;
      const feature: any = new Feature({
        geometry: new geom.Point([Number(lng || 0), Number(lat || 0)]),
      });
      feature.userData = icon;
      feature.groupId = groupId;
      return feature;
    });
    if (!newList.length) return false;
    const cluseterSource = new source.Vector({ wrapX: false });
    const cluseterLayer = new layer.Vector({
      source: new source.Cluster({
        source: cluseterSource,
        distance,
      }),
      visible: defaultVisible,
      zIndex: zIndex,
      style: feature =>
        !styleFn
          ? createClusterStyle(feature, styleOpts)
          : styleFn(feature, styleOpts),
    });
    cluseterSource?.addFeatures(newList);
    mapRef?.current?.addLayer(cluseterLayer);
    clusterFeatureLayerRef?.current?.set(groupId, cluseterLayer);
    return newList;
  };

  const clusterFeatureRemove = (groupId: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    let status = true;
    if (Array.isArray(groupId)) {
      const groupIds = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getClusterGroupId(it));
      groupIds?.forEach(it => {
        const layerArea = clusterFeatureLayerRef?.current?.get(it);
        if (layerArea) {
          mapRef?.current?.removeLayer(layerArea);
          clusterFeatureLayerRef?.current?.delete(it);
        } else {
          status = false;
        }
      });
    } else {
      const key = GetId.getClusterGroupId(groupId);
      const layerArea = clusterFeatureLayerRef?.current?.get(key);
      if (layerArea) {
        mapRef?.current?.removeLayer(layerArea);
        clusterFeatureLayerRef?.current?.delete(key);
      } else {
        status = false;
      }
    }

    return status;
  };

  const clusterFeatureSetVisible = (
    groupId: string | string[],
    visible: boolean
  ) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!groupId) return false;
    let status = true;
    if (Array.isArray(groupId)) {
      const keys = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getClusterGroupId(it));
      keys?.forEach((it: string) => {
        const areaMap = clusterFeatureLayerRef?.current?.get(it);
        if (areaMap) {
          areaMap?.setVisible(visible);
        } else {
          status = false;
        }
      });
    } else {
      const key = GetId.getClusterGroupId(groupId);
      const areaMap = clusterFeatureLayerRef?.current?.get(key);
      if (areaMap) {
        areaMap?.setVisible(visible);
      } else {
        status = false;
      }
    }

    return status;
  };

  const clusterFeatureSetToggetVisible = (groupId: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!groupId) return false;
    let status = true;
    if (Array.isArray(groupId)) {
      const keys = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getClusterGroupId(it));
      keys?.forEach((it: string) => {
        const areaMap = clusterFeatureLayerRef?.current?.get(it);
        if (areaMap) {
          areaMap?.setVisible(!areaMap?.getVisible());
        } else {
          status = false;
        }
      });
    } else {
      const key = GetId.getClusterGroupId(groupId);
      const areaMap = clusterFeatureLayerRef?.current?.get(key);
      if (areaMap) {
        areaMap?.setVisible(!areaMap?.getVisible());
      } else {
        status = false;
      }
    }

    return status;
  };

  const isClusterLayerVisible = (groupId: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!groupId) return false;
    let status = true;
    if (Array.isArray(groupId)) {
      const keys = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getClusterGroupId(it));
      keys?.forEach((it: string) => {
        const layer = clusterFeatureLayerRef?.current?.get(it);
        if (!layer) status = false;
      });

      return status;
    }
    const key = GetId.getClusterGroupId(groupId);
    const layer = clusterFeatureLayerRef?.current?.get(key);
    return !!layer;
  };

  const clusterFeatureClear = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const iconList = Array.from(clusterFeatureLayerRef?.current?.values());
    iconList?.forEach((it: any) => {
      mapRef?.current?.removeLayer(it);
    });
    clusterFeatureLayerRef?.current?.clear();
    return true;
  };

  const clusterFeatureFocus = (
    groupId: string | string[],
    padding?: [number, number, number, number]
  ) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const myPadding = padding || [100, 100, 100, 100];
    if (Array.isArray(groupId)) {
      const groupIds = groupId
        ?.filter((it: any) => it)
        ?.map((it: string) => GetId.getClusterGroupId(it));
      const mainList1 = groupIds?.map((id: string) => {
        const iconFeature = clusterFeatureLayerRef?.current?.get(id) as any;
        const data = iconFeature?.userData;
        return data;
      });
      const wkt1 = setFocus(mainList1);
      return setFit(wkt1!, myPadding);
    }
    const groupIds = [groupId]
      ?.filter((it: any) => it)
      ?.map((it: string) => GetId.getClusterGroupId(it));
    const mainList = groupIds?.map((id: string) => {
      const iconFeature = clusterFeatureLayerRef?.current?.get(id) as any;
      const data = iconFeature?.userData;
      return data;
    });
    const wkt = setFocus(mainList);
    return setFit(wkt!, myPadding);
  };

  const clusterFeatureCheck = (groupId: string) => {
    const key = GetId.getClusterGroupId(groupId);
    const layer = clusterFeatureLayerRef?.current?.get(key);
    return !!layer;
  };
  // ------------------------------

  // 浮动层-------------------------
  const overlayerAdd = (opts: OLMAP.OverlayFeaturesType.AddOverlayerOpts) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const { positioning = 'bottom-center', element, id } = opts;
    const overlay = mapRef?.current?.getOverlayById(id);
    if (overlay) return false;
    if (!element) {
      console.warn('html元素错误！');
      return false;
    }
    const eleStyle: any = element.style;
    if ((eleStyle || eleStyle?.display) === 'none') {
      element.style.display = '';
    }
    const popup = new ol.Overlay(
      Object.assign(
        {},
        { ...opts! },
        {
          positioning: positioning,
          element: element,
        }
      )
    );
    mapRef?.current?.addOverlay(popup);
    return popup;
  };

  const overlayerRemove = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    let status = true;
    const overlay = mapRef?.current?.getOverlayById(id);
    if (overlay) {
      mapRef?.current?.removeOverlay(overlay);
    } else {
      status = false;
    }

    return status;
  };

  const overlayerSetVisible = (id: string, visible: boolean) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!id) return false;
    let status = true;
    const popup = mapRef?.current?.getOverlayById(id);
    if (!popup) {
      return false;
    }
    if (visible) {
      const position = positionMap?.current?.get(id);
      if (!popup?.getPosition() && position) {
        popup?.setPosition(position);
      }
    } else {
      positionMap?.current?.set(id, popup.getPosition());
      popup?.setPosition(undefined as any);
    }

    return status;
  };

  const overlayerSetToggetVisible = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!id) return false;
    const popup = mapRef?.current?.getOverlayById(id);
    if (!popup) {
      return false;
    }
    const popupPosition = popup?.getPosition();

    return overlayerSetVisible(id, !popupPosition);
  };

  const isOverlayerLayerVisible = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!id) return false;
    const popup = mapRef?.current?.getOverlayById(id);
    return !!popup;
  };

  const overlayerClear = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const overlays = mapRef?.current?.getOverlays();
    overlays
      ?.getArray()
      ?.filter((itm: any) => !itm?.id?.includes('_change_key')) // 切换浮动层不清除
      ?.forEach(itm => {
        overlays?.remove(itm);
      });
    positionMap?.current?.clear();
    return true;
  };

  const overlayerSetPosition = (
    id: string,
    coordinate: [number, number] | number[]
  ) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    const [lng, lat] = coordinate;
    let popup = mapRef?.current?.getOverlayById(id);
    if (!popup) {
      return false;
    }
    if (lng && lat) {
      popup.setPosition([lng, lat]);
      return true;
    }
    return false;
  };
  // ------------------------------

  // 浮动层切换---------------------
  const overlayChangeAdd = (
    opts: OLMAP.OverlayChangeType.OverlayChangeOpts
  ) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    const { pointerList, time = 4, getChangeDataFn } = opts;
    const addOverlayerOpts = loadshGet.omit(opts, [
      'pointerList',
      'time',
      'getChangeDataFn',
    ]);
    let id = GetId.overlayChangeGetId(addOverlayerOpts.id);
    if (mapRef?.current?.getOverlayById(id)) return false;
    let newList = filterLngLatByList(pointerList);
    if (!newList.length) return;
    const { lng, lat } = newList?.[0] || [];
    const position: [number, number] = [lng, lat];
    let overlay = overlayerAdd(
      Object.assign({}, Object.assign({}, addOverlayerOpts), { position, id })
    );
    if (!overlay) return false;
    if (newList.length < 2) return false;
    changeterMap?.current?.set(id, {
      lisenter: createInterval(id, newList, time, getChangeDataFn),
      list: newList,
      getChangeDataFn: getChangeDataFn,
      time: time,
    });
    return true;
  };

  const overlayChangeRemove = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    let newId = GetId.overlayChangeGetId(id);
    let changeter = changeterMap?.current?.get(newId);
    if (changeter.lisenter) {
      clearTimeout(changeter.lisenter);
    }
    changeterMap?.current?.delete(newId);
    overlayerRemove(newId);
    return true;
  };

  const overlayChangeSetVisible = (id: string, visible: boolean) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    let newId = GetId.overlayChangeGetId(id);
    let status = visible ? overlayChangeStart(id) : overlayChangeStop(id);
    overlayerSetVisible(newId, visible);
    return status;
  };

  const overlayChangeSetToggetVisible = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    let newId = GetId.overlayChangeGetId(id);
    const popup = mapRef?.current?.getOverlayById(newId);
    const popupPosition = popup?.getPosition();

    return overlayChangeSetVisible(id, !popupPosition);
  };

  const isOverlayerChangeLayerVisible = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!id) return false;
    let newId = GetId.overlayChangeGetId(id);
    const popup = mapRef?.current?.getOverlayById(newId);
    return !!popup;
  };

  const overlayChangeClear = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    let overlays = mapRef?.current?.getOverlays();
    overlays.getArray().forEach((item: any) => {
      if (item?.id || item?.id?.includes('_change_key')) {
        overlayChangeRemove(item.id.replace(/_change_key/, ''));
      }
    });
    changeterMap?.current?.clear();
    return true;
  };

  const overlayChangeStart = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    let newId = GetId.overlayChangeGetId(id);
    let changeter = changeterMap?.current?.get(newId);
    if (!changeter) {
      console.warn(
        '\u5F00\u59CB\u5931\u8D25id:' + id + '\uFF0C\u672A\u627E\u5230'
      );
      return false;
    }
    if (changeter.lisenter) return false;
    const { list, time, getChangeDataFn } = changeter;
    changeterMap?.current?.set(
      newId,
      Object.assign({}, Object.assign({}, changeter), {
        lisenter: createInterval(newId, list, time, getChangeDataFn),
      })
    );
    return true;
  };

  const overlayChangeStop = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    let newId = GetId.overlayChangeGetId(id);
    let changeter = changeterMap?.current?.get(newId);
    if (!changeter) {
      console.warn(
        '\u5F00\u59CB\u5931\u8D25id:' + id + '\uFF0C\u672A\u627E\u5230'
      );
      return false;
    }
    const { lisenter } = changeter;
    if (!lisenter) return false;
    clearTimeout(lisenter);
    changeterMap?.current?.set(
      newId,
      Object.assign({}, Object.assign({}, changeter), { lisenter: undefined })
    );
    return true;
  };

  const createInterval = (
    id: any,
    list: any,
    time: any,
    getChangeDataFn: any
  ) => {
    let index = 0;
    const timer = setInterval(function() {
      index += 1;
      if (index >= list.length) {
        index = 0;
      }
      let _a = list[index],
        lng = _a.lng,
        lat = _a.lat;
      if (getChangeDataFn) {
        getChangeDataFn(list[index]);
      }
      let setPoint = overlayerSetPosition(id, [lng, lat]);
      if (!setPoint) {
        clearInterval(timer);
      }
    }, 1000 * time);
    return timer;
  };
  // ------------------------------

  // 运动轨迹----------------------
  const TrajectoryAdd = (opts: OLMAP.TrajectoryType.AddTrajectoryOpts) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    const {
      id,
      list,
      styleFn,
      borderColor = '#f00',
      borderSize = 4,
      getCoordinate,
      interpolation = 40,
      isLock = true,
      zIndex = 30,
    } = opts;
    if (!id || !(list === null || list === void 0 ? void 0 : list.length))
      return false;
    if (!isLock && trajectoryLayerMap?.current?.has(id)) {
      TrajectoryRemove(id);
    }
    if (isLock && trajectoryLayerMap?.current?.has(id)) return false;
    const trSource = new source.Vector();
    const trGeometry = new geom.LineString([proj.fromLonLat([0, 0])]);
    const LineStringFeature: any = new Feature(trGeometry);
    LineStringFeature.groupId = id;
    trSource?.addFeature(LineStringFeature);
    const vectorLayer = new layer.Vector({
      source: trSource,
      zIndex: zIndex,
      style: !styleFn
        ? createLineStyle({ borderColor: borderColor, borderSize: borderSize })
        : styleFn,
    });
    trajectoryLayerMap?.current?.set(id, vectorLayer);
    trajectoryOptsMap?.current?.set(id, opts);
    mapRef?.current?.addLayer(vectorLayer);
    const coordinateList = coordinateMoreChildren(
      filterLngLatByList(list),
      interpolation
    );
    let index = -1;
    const coordinate = coordinateList.slice(0, 1);
    const interval = setInterval(function() {
      index += 1;
      if (coordinateList[index]) {
        coordinate.push(coordinateList[index]);
        if (getCoordinate) getCoordinate(coordinateList[index]);
        trGeometry.setCoordinates(coordinate);
      } else {
        clearInterval(interval);
      }
    }, 50);
    return true;
  };

  const TrajectoryRemove = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    if (!id) return false;
    let status = true;
    if (trajectoryLayerMap?.current?.has(id)) {
      const lay = trajectoryLayerMap?.current?.get(id);
      mapRef?.current?.removeLayer(lay!);
      trajectoryLayerMap?.current?.delete(id);
      trajectoryOptsMap?.current?.delete(id);
    } else {
      console.warn(
        '\u5220\u9664\u5931\u8D25id:' +
          id +
          '\uFF0C\u672A\u627E\u5230 --- UseTrajectory'
      );
      status = false;
    }
    return status;
  };

  const TrajectorySetVisible = (id: string, visible: boolean) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    if (!id) return false;
    let status = true;
    if (trajectoryLayerMap?.current?.has(id)) {
      const lay = trajectoryLayerMap?.current?.get(id);
      lay?.setVisible(visible);
    } else {
      status = false;
    }
    return status;
  };

  const TrajectorySetToggetVisible = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    if (!id) return false;
    let status = true;
    if (trajectoryLayerMap?.current?.has(id)) {
      const lay = trajectoryLayerMap?.current?.get(id);
      lay?.setVisible(!lay?.getVisible());
    } else {
      status = false;
    }
    return status;
  };

  const isTrajectoryLayerVisible = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化地图');
      return false;
    }
    if (!id) return false;
    const popup = trajectoryLayerMap?.current?.get(id);
    return !!popup;
  };

  const TrajectoryClear = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    Array.from(trajectoryLayerMap?.current?.values())?.forEach((it: any) => {
      if (!it) return;
      mapRef?.current?.removeLayer(it);
    });
    trajectoryLayerMap?.current?.clear();
    trajectoryOptsMap?.current?.clear();
    return true;
  };

  const TrajectoryRestart = (id: string) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    let opts = trajectoryOptsMap?.current?.get(id);
    if (!opts) return false;
    TrajectoryRemove(id);
    TrajectoryAdd(opts);
    return true;
  };
  // -----------------------------

  // 事件监听----------------
  const addEventListener = (
    key: string,
    type: OLMAP.UseEventType.EventType,
    listener: (e: OLMAP.UseEventType.MapEvent) => void
  ) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return;
    }
    if (!type || !listener) return;
    if (listenerRef?.current?.[type]?.[key]) return;
    const newListener = (e: any) => {
      // console.log('事件监听', e, mapRef?.current);
      const pixel = e.pixel || e.pixel_;
      const evt1: any = pixel
        ? [pixel?.[0] / xScale, pixel?.[1] / yScale]
        : undefined;
      // if (!evt1) {
      //   const [, , , , lng, lat] = e?.frameState?.pixelToCoordinateTransform;
      //   evt1 = (lng && lat) ? mapRef?.current?.getPixelFromCoordinate([lng, lat]) : undefined;
      // }
      let features = evt1
        ? mapRef?.current?.getFeaturesAtPixel(evt1)
        : undefined;
      let mapCenter = mapRef?.current?.getView()?.getCenter();
      let clickCoordinate = evt1
        ? mapRef?.current?.getCoordinateFromPixel(evt1)
        : undefined;
      const iconFeatures: OLMAP.UseEventType.FeaturesType[] = [];
      const areaFeatures: OLMAP.UseEventType.FeaturesType[] = [];
      const clusterFeatures: OLMAP.UseEventType.FeaturesType[] = [];
      features?.forEach((it: any) => {
        const groupId = it?.groupId;
        if (GetId.checkIconGroupId(groupId)) {
          iconFeatures?.push(it);
        }
        if (GetId.checkAreaGroupId(groupId)) {
          areaFeatures?.push(it);
        }
        if (
          it.getProperties() &&
          it.getProperties()?.features !== undefined &&
          it.getProperties()?.features?.length > 0
        ) {
          clusterFeatures?.push(it);
        }
      });
      const iconZIndex =
        iconFeatureLayerRef?.current
          ?.get((iconFeatures?.[0] as any)?.groupId)
          ?.getZIndex() || -1;
      const areaZIndex =
        areaMapLayerRef?.current
          ?.get((areaFeatures?.[0] as any)?.groupId)
          ?.getZIndex() || -1;
      const clusterZIndex =
        clusterFeatureLayerRef?.current
          ?.get(
            (clusterFeatures?.[0]?.getProperties()?.features?.[0] as any)
              ?.groupId
          )
          ?.getZIndex() || -1;

      const sortObj = {
        [iconZIndex]: 'icon',
        [areaZIndex]: 'area',
        [clusterZIndex]: 'cluster',
      };
      const firstOne = Object.keys(sortObj)?.sort(
        (a: any, b: any) => Number(b) - Number(a)
      )?.[0];
      const type = sortObj?.[Number(firstOne)];

      let clickFeatures: any[] = [];
      switch (type) {
        case 'icon':
          clickFeatures = iconFeatures;
          break;
        case 'area':
          clickFeatures = areaFeatures;
          break;
        case 'cluster':
          clickFeatures = clusterFeatures?.[0]?.getProperties()?.features;
          break;
        default:
          break;
      }
      // console.log('clickFeatures', clickFeatures);

      listener(
        Object.assign({}, Object.assign({}, e), {
          iconFeatures,
          areaFeatures,
          // 聚合点位中普通点位列表数据
          clusterFeatures,
          clickFeatures,
          clickType: type,
          features: features,
          mapCenter: mapCenter,
          clickCoordinate: clickCoordinate,
        })
      );
    };
    mapRef?.current?.addEventListener(type, newListener);
    listenerRef.current[type] = Object.assign(
      {},
      Object.assign({}, listenerRef?.current?.[type]),
      { [key]: newListener }
    );
  };

  // 不传key默认清除type下所有事件
  const removeEventListener = (
    type: OLMAP.UseEventType.EventType,
    key?: string | string[] | undefined
  ) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return;
    }
    if (!type) return;
    if (!key) {
      Object.values(listenerRef?.current?.[type] || {}).forEach(function(e) {
        mapRef?.current?.removeEventListener(type, e as any);
      });
      listenerRef.current[type] = undefined;
      return;
    }
    if (Array.isArray(key)) {
      const keys = key?.filter((it: any) => it);
      keys?.forEach(function(str) {
        let removeFn = listenerRef?.current?.[type]?.[str];
        if (removeFn) {
          mapRef?.current?.removeEventListener(type, removeFn as any);
          delete listenerRef?.current?.[type]?.[str];
        }
      });
      return;
    }
    let removeFn1 = listenerRef?.current?.[type]?.[key];
    if (removeFn1) {
      mapRef?.current?.removeEventListener(type, removeFn1 as any);
      delete listenerRef?.current?.[type]?.[key];
    }
  };

  const clearEventListener = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return;
    }
    Object.keys(listenerRef?.current).forEach(type => {
      removeEventListener(type as any);
    });
  };
  // -------------------------

  // 地图交互事件 点位选中样式---------------------------
  const addInteractionSelect = (
    opts: OLMAP.UseInteractionType.AddInteractionSelectOpts
  ) => {
    const {
      interactionKey,
      layerKeys,
      eventType = 'select',
      eventCallback,
      styleFn,
    } = opts;
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    if (interactionSelectMap?.current?.has(interactionKey)) {
      const interaction = interactionSelectMap?.current?.get(interactionKey);
      mapRef?.current?.removeInteraction(interaction);
      interactionSelectMap?.current?.delete(interactionKey);
    }
    const layers: any[] = layerKeys
      ?.map((itm: string) => {
        const areaLayer = areaMapLayerRef?.current?.get(
          GetId.getAreaGroupId(itm)
        );
        const iconLayer = iconFeatureLayerRef?.current?.get(
          GetId.getIconGroupId(itm)
        );
        const clusterLayer = clusterFeatureLayerRef?.current?.get(
          GetId.getClusterGroupId(itm)
        );
        if (clusterLayer) return clusterLayer;
        if (areaLayer) return areaLayer;
        if (iconLayer) return iconLayer;
        return undefined;
      })
      ?.filter((it: any) => it);

    const selectInteraction = new olInteraction.Select({
      multi: false,
      layers,
      style: !styleFn
        ? (feature: OLMAP.UseEventType.FeaturesType) => {
            const realFeatures = feature?.getProperties()?.features;
            const size = realFeatures?.length;
            console.log('select', size);
            return new olStyle.Style({
              image: new olStyle.Circle({
                radius: 20,
                fill: new olStyle.Fill({
                  color: '#CCFFFF',
                }),
              }),
              // text,
              text: new olStyle.Text({
                font: '18px DINPro-Bold, DINPro', // 默认这个字体，可以修改成其他的，格式和css的字体设置一样
                text: `${size || 1}`,
                fill: new olStyle.Fill({
                  // color: '#333333',
                  color: 'rgba(60, 107, 251, 1)',
                }),
                offsetY: 2,
              }),
            });
          }
        : styleFn,
    });
    if (eventType && eventCallback) {
      selectInteraction?.on(eventType as any, eventCallback);
    }
    interactionSelectMap?.current?.set(interactionKey, selectInteraction);
    mapRef?.current?.addInteraction(selectInteraction);
    return selectInteraction;
  };

  // 地图交互绘制图形
  const addInteractionDraw = (
    opts: OLMAP.UseInteractionType.AddInteractionDrawOpts
  ) => {
    const {
      interactionKey,
      style,
      drawType = 'Polygon',
      eventList,
      snapOpts,
      modifyOpts,
    } = opts;
    const otherOpts = loadshGet.omit(opts, [
      'interactionKey',
      'style',
      'drawType',
      'eventType',
      'eventCallback',
    ]);
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    if (interactionSelectMap?.current?.has(interactionKey)) {
      const interaction = interactionSelectMap?.current?.get(interactionKey);
      mapRef?.current?.removeInteraction(interaction);
      interactionSelectMap?.current?.delete(interactionKey);
    }
    const drawSource = new source.Vector();
    const drawVector = new layer.Vector({
      source: drawSource,
      style: new olStyle.Style({
        fill: new olStyle.Fill({
          color: 'rgba(255, 255, 255, 0.2)',
        }),
        stroke: new olStyle.Stroke({
          color: '#ffcc33',
          width: 2,
        }),
        image: new olStyle.Circle({
          radius: 7,
          fill: new olStyle.Fill({
            color: '#ffcc33',
          }),
        }),
      }),
      zIndex: 9999,
    });

    const drawInteraction = new olInteraction.Draw({
      source: drawSource,
      type: drawType,
      style: !style ? DrawDefaultStyle?.[drawType] : style,
      ...otherOpts,
    });

    eventList?.forEach(({ eventType, eventCallback }: any) => {
      if (eventType && eventCallback) {
        drawInteraction?.on(eventType as any, eventCallback);
      }
    });

    if (snapOpts?.isAdd) {
      const snapEventList = snapOpts?.eventList || [];
      const opts = loadshGet.omit(snapOpts, ['isAdd', 'eventList']);
      const drawSnap = new olInteraction.Snap({ source: drawSource, ...opts });

      snapEventList?.forEach(({ eventType, eventCallback }: any) => {
        if (eventType && eventCallback) {
          drawSnap?.on(eventType as any, eventCallback);
        }
      });
      interactionSnapMap?.current?.set(interactionKey, drawSnap);
      mapRef?.current.addInteraction(drawSnap);
    }
    if (modifyOpts?.isAdd) {
      const modifyEventList = modifyOpts?.eventList || [];
      const opts = loadshGet.omit(modifyOpts, ['isAdd', 'eventList']);
      const drawModify = new olInteraction.Modify({
        source: drawSource,
        ...opts,
      });

      modifyEventList?.forEach(({ eventType, eventCallback }: any) => {
        if (eventType && eventCallback) {
          drawModify?.on(eventType as any, eventCallback);
        }
      });
      interactionModifyMap?.current?.set(interactionKey, drawModify);
      mapRef?.current.addInteraction(drawModify);
    }

    interactionSelectMap?.current?.set(interactionKey, drawInteraction);
    interactionDrawLayerMap?.current?.set(interactionKey, drawVector);
    mapRef?.current?.addInteraction(drawInteraction);
    mapRef?.current?.addLayer(drawVector);
    return drawInteraction;
  };

  const removeInteraction = (interactionKey: string | string[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    let status = true;
    if (Array.isArray(interactionKey)) {
      const keys = interactionKey?.filter((it: any) => it);
      keys?.forEach((it: string) => {
        const interaction = interactionSelectMap?.current?.get(it);
        const drawLayer = interactionDrawLayerMap?.current.get(it);
        const modifyLayer = interactionModifyMap?.current?.get(it);
        const snapLayer = interactionSnapMap?.current?.get(it);
        if (!interaction && !modifyLayer && !snapLayer && !drawLayer) {
          status = false;
          return;
        }
        if (interaction) {
          mapRef?.current?.removeInteraction(interaction);
          interactionSelectMap?.current?.delete(it);
        }

        if (drawLayer) {
          mapRef?.current?.removeLayer(drawLayer);
          interactionDrawLayerMap?.current?.delete(it);
        }

        if (modifyLayer) {
          mapRef?.current?.removeInteraction(modifyLayer);
          interactionModifyMap?.current?.delete(it);
        }

        if (snapLayer) {
          mapRef?.current?.removeInteraction(snapLayer);
          interactionSnapMap?.current?.delete(it);
        }
      });
    } else {
      const interaction = interactionSelectMap?.current?.get(interactionKey);
      const drawLayer = interactionDrawLayerMap?.current.get(interactionKey);
      const modifyLayer = interactionModifyMap?.current?.get(interactionKey);
      const snapLayer = interactionSnapMap?.current?.get(interactionKey);
      if (!interaction && !modifyLayer && !snapLayer && !drawLayer) {
        status = false;
      }
      if (interaction) {
        mapRef?.current?.removeInteraction(interaction);
        interactionSelectMap?.current?.delete(interactionKey);
      }

      if (drawLayer) {
        mapRef?.current?.removeLayer(drawLayer);
        interactionDrawLayerMap?.current?.delete(interactionKey);
      }

      if (modifyLayer) {
        mapRef?.current?.removeInteraction(modifyLayer);
        interactionModifyMap?.current?.delete(interactionKey);
      }

      if (snapLayer) {
        mapRef?.current?.removeInteraction(snapLayer);
        interactionSnapMap?.current?.delete(interactionKey);
      }
    }

    return status;
  };

  const clearInteraction = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return false;
    }
    const iconList = Array.from(interactionSelectMap?.current?.values());
    iconList?.forEach((it: any) => {
      mapRef?.current?.removeInteraction(it);
    });
    interactionSelectMap?.current?.clear();

    const modifyList = Array.from(interactionModifyMap?.current?.values());
    modifyList?.forEach((it: any) => {
      mapRef?.current?.removeInteraction(it);
    });
    interactionModifyMap?.current?.clear();

    const snapList = Array.from(interactionSnapMap?.current?.values());
    snapList?.forEach((it: any) => {
      mapRef?.current?.removeInteraction(it);
    });
    interactionSnapMap?.current?.clear();

    const drawLayerList = Array.from(
      interactionDrawLayerMap?.current?.values()
    );
    drawLayerList?.forEach((it: any) => {
      mapRef?.current?.removeLayer(it);
    });
    interactionDrawLayerMap?.current?.clear();
    return true;
  };
  // ---------------------

  // 控制器 ----------------------
  const ControlsAdd = (control: Keys<TOOLSTYPE> | Keys<TOOLSTYPE>[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return;
    }
    if (!control) return;
    if (Array.isArray(control)) {
      control
        ?.filter((it: any) => it)
        ?.map(str => {
          mapRef?.current?.addControl(TOOLSDATA[str]);
        });
    } else {
      mapRef?.current?.addControl(TOOLSDATA[control]);
    }
  };

  const ControlsRemove = (control: Keys<TOOLSTYPE> | Keys<TOOLSTYPE>[]) => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return;
    }
    if (!control) return;
    if (Array.isArray(control)) {
      control
        ?.filter((it: any) => it)
        ?.map(str => {
          mapRef?.current?.removeControl(TOOLSDATA[str]);
        });
    } else {
      mapRef?.current?.removeControl(TOOLSDATA[control]);
    }
  };

  const ControlsClear = () => {
    if (!mapRef?.current) {
      console.warn('请先初始化！');
      return;
    }
    mapRef?.current?.getControls()?.clear();
  };

  // ----------------------

  useImperativeHandle(ref, () => ({
    BaseMapRef: { mapRef },
    UseFormatArea,
    UseCoordinate,
    UseAllMapLayer: {
      setLayerVisible,
      setLayerToggetVisible,
      removeLayer,
    },
    UseView: {
      setCenter,
      setZoom,
      setFit,
      getCenter,
      getZoom,
    },
    // 地图底图
    UseTileLayer: {
      mapBaseLayerRef,
      setWmtsLayer,
      setTDTLayer,
      setTDTLWLayer,
      setGaoDeLayer,
      setBaiduLayer,
      setYXLayer,
      setUpYouLayer,
      setTileArcGISRestLayer,
      removeLayer: removeBaseLayer,
      setVisible: setTileLayerVisible,
      setToggetVisible: setTileLayerToggetVisible,
      isVisible: isTileLayerVisible,
      clear: tileLayerClear,
      hideAll: tileLayerHideAll,
      showAll: tileLayerShowAll,
    },
    // 区域图层加载，加载边界数据
    UseAreaFeatures: {
      areaMapLayerRef,
      add: areaFeaturesAdd,
      remove: areaFeaturesRemove,
      setVisible: setAreaFeaturesVisible,
      setToggetVisible: setAreaFeaturesToggetVisible,
      isVisible: isAreaLayerVisible,
      clear: clearAreaFeatures,
    },
    // 点位图层加载
    UseIconFeatures: {
      iconFeatureLayerRef,
      add: iconFeaturesAdd,
      remove: iconFeaturesRemove,
      setVisible: iconFeaturesSetVisible,
      isVisible: isIconLayerVisible,
      setToggetVisible: iconFeaturesSetToggetVisible,
      clear: iconFeaturesClear,
      focus: iconFeatureFocus,
      setPosition: iconFeatureSetPosition,
      check: iconFeatureCheck,
    },
    // 聚合图标图层
    UseClusterFeatures: {
      clusterFeatureLayerRef,
      add: clusterFeatureAdd,
      remove: clusterFeatureRemove,
      setVisible: clusterFeatureSetVisible,
      setToggetVisible: clusterFeatureSetToggetVisible,
      isVisible: isClusterLayerVisible,
      clear: clusterFeatureClear,
      focus: clusterFeatureFocus,
      check: clusterFeatureCheck,
    },
    // 图片图层
    // 浮动层
    UseOverlayFeatures: {
      positionMap,
      add: overlayerAdd,
      remove: overlayerRemove,
      setVisible: overlayerSetVisible,
      setToggetVisible: overlayerSetToggetVisible,
      isVisible: isOverlayerLayerVisible,
      clear: overlayerClear,
      setPosition: overlayerSetPosition,
    },
    // 浮动层切换
    UseOverlayChange: {
      add: overlayChangeAdd,
      remove: overlayChangeRemove,
      setVisible: overlayChangeSetVisible,
      setToggetVisible: overlayChangeSetToggetVisible,
      isVisible: isOverlayerChangeLayerVisible,
      clear: overlayChangeClear,
      stop: overlayChangeStop,
      start: overlayChangeStart,
    },
    // 运动轨迹
    UseTrajectory: {
      trajectoryLayerMap,
      add: TrajectoryAdd,
      remove: TrajectoryRemove,
      setVisible: TrajectorySetVisible,
      setToggetVisible: TrajectorySetToggetVisible,
      isVisible: isTrajectoryLayerVisible,
      clear: TrajectoryClear,
      restart: TrajectoryRestart,
    },
    // 事件
    UseEvent: {
      listenerRef,
      addEventListener,
      removeEventListener,
      clearEventListener,
    },
    UseStyle: {
      createClusterStyle,
      createAreaStyle,
      createIconStyle,
      createLineStyle,
    },
    UseInteraction: {
      addSelect: addInteractionSelect,
      addDraw: addInteractionDraw,
      remove: removeInteraction,
      clear: clearInteraction,
    },
    UseControls: {
      add: ControlsAdd,
      remove: ControlsRemove,
      clear: ControlsClear,
    },
  }));

  return (
    <div id={mapId || 'map'} style={{ width: '100%', height: '100%' }}>
      {children}
    </div>
  );
};
export default memo(forwardRef<OlMyMapRef, OlMyMapProps>(OlMyMap));
