/* eslint-disable @typescript-eslint/typedef */

/**
 * @原算法 https://www.jianshu.com/p/57ca061f3987
 * @根据该作者的修改成JS版的
 * @time 2019-7-17 09:58:42
 * @description bd09 转WGS84,精准度高
 * */
const CoordinateUtil = {
  x_pi: 3.14159265358979324 * 3000.0 / 180.0,
  // pai
  pi: 3.1415926535897932384626,
  // 离心率
  ee: 0.00669342162296594323,
  // 长半轴
  a: 6378245.0,
  // 百度转国测局
  bd09togcj02(bd_lon: number, bd_lat: number) {
    const x = bd_lon - 0.0065;
    const y = bd_lat - 0.006;
    const z = Math.sqrt(x * x + y * y) - 0.00002 * Math.sin(y * this.x_pi);
    const theta = Math.atan2(y, x) - 0.000003 * Math.cos(x * this.x_pi);
    const gg_lng = z * Math.cos(theta);
    const gg_lat = z * Math.sin(theta);
    // return {
    // 	lng: gg_lng,
    // 	lat: gg_lat
    // }
    return [gg_lng, gg_lat]
  },
  // 国测局转百度
  gcj02tobd09(lng: number, lat: number) {
    const z = Math.sqrt(lng * lng + lat * lat) + 0.00002 * Math.sin(lat * this.x_pi);
    const theta = Math.atan2(lat, lng) + 0.000003 * Math.cos(lng * this.x_pi);
    const bd_lng = z * Math.cos(theta) + 0.0065;
    const bd_lat = z * Math.sin(theta) + 0.006;
    // return {
    // 	lng: bd_lng,
    // 	lat: bd_lat
    // };
    return [bd_lng, bd_lat]
  },
  // 国测局转84
  gcj02towgs84(lng: number, lat: number) {
    let dlat = this.transformlat(lng - 105.0, lat - 35.0);
    let dlng = this.transformlng(lng - 105.0, lat - 35.0);
    const radlat = lat / 180.0 * this.pi;
    let magic = Math.sin(radlat);
    magic = 1 - this.ee * magic * magic;
    const sqrtmagic = Math.sqrt(magic);
    dlat = (dlat * 180.0) / ((this.a * (1 - this.ee)) / (magic * sqrtmagic) * this.pi);
    dlng = (dlng * 180.0) / (this.a / sqrtmagic * Math.cos(radlat) * this.pi);
    const mglat = lat + dlat;
    const mglng = lng + dlng;
    // return {
    // 	lng: lng * 2 - mglng,
    // 	lat: lat * 2 - mglat
    // };
    return [lng * 2 - mglng, lat * 2 - mglat]
  },
  // 经度转换
  transformlat(lng: number, lat: number) {
    let ret = -100.0 + 2.0 * lng + 3.0 * lat + 0.2 * lat * lat + 0.1 * lng * lat + 0.2 * Math.sqrt(Math.abs(lng));
    ret += (20.0 * Math.sin(6.0 * lng * CoordinateUtil.pi) + 20.0 * Math.sin(2.0 * lng * CoordinateUtil.pi)) * 2.0 / 3.0;
    ret += (20.0 * Math.sin(lat * CoordinateUtil.pi) + 40.0 * Math.sin(lat / 3.0 * CoordinateUtil.pi)) * 2.0 / 3.0;
    ret += (160.0 * Math.sin(lat / 12.0 * CoordinateUtil.pi) + 320 * Math.sin(lat * CoordinateUtil.pi / 30.0)) * 2.0 / 3.0;
    return ret;
  },
  // 纬度转换
  transformlng(lng: number, lat: number) {
    let ret = 300.0 + lng + 2.0 * lat + 0.1 * lng * lng + 0.1 * lng * lat + 0.1 * Math.sqrt(Math.abs(lng));
    ret += (20.0 * Math.sin(6.0 * lng * CoordinateUtil.pi) + 20.0 * Math.sin(2.0 * lng * CoordinateUtil.pi)) * 2.0 / 3.0;
    ret += (20.0 * Math.sin(lng * CoordinateUtil.pi) + 40.0 * Math.sin(lng / 3.0 * CoordinateUtil.pi)) * 2.0 / 3.0;
    ret += (150.0 * Math.sin(lng / 12.0 * CoordinateUtil.pi) + 300.0 * Math.sin(lng / 30.0 * CoordinateUtil.pi)) * 2.0 / 3.0;
    return ret;
  },
  getWgs84xy(x: number, y: number) {
    // 先转 国测局坐标
    const doubles_gcj = CoordinateUtil.bd09togcj02(x, y); // （x 117.   y 36. ）
    // 国测局坐标转wgs84
    const doubles_wgs84 = CoordinateUtil.gcj02towgs84(doubles_gcj[0], doubles_gcj[1]);
    // 返回 纠偏后 坐标

    return doubles_wgs84;
  }
}

export default CoordinateUtil;