import { bd09togcj02, bd09towgs84, gcj02tobd09, gcj02towgs84, out_of_china, wgs84togcj02 } from '../utils/pointerTransfroms';
/**
 * 点位处理工具
 */
declare type UseCoordinate = {
  /**
   * 4326转3857
   * @param coordinate 点位
   */
  set4326to3857: (coordinate: [number, number]) => number[] | null;
  /**
   * 3857转4326
   * @param coordinate 点位
   */
  set3857to4326: (coordinate: [number, number]) => number[] | null;
  /**
   * 百度坐标系 (BD-09) 与 火星坐标系 (GCJ-02)的转换
   * 即 百度 转 谷歌、高德
   * @param bd_lon
   * @param bd_lat
   * @returns [lng,lat]
   */
  bd09togcj02: typeof bd09togcj02;
  /**
   * 百度坐标转wgs84
   * @param {*} lng
   * @param {*} lat
   * @returns [lng,lat]
   */
  bd09towgs84: typeof bd09towgs84;
  /**
   * 火星坐标系 (GCJ-02) 与百度坐标系 (BD-09) 的转换
   * 即谷歌、高德 转 百度
   * @param lng
   * @param lat
   * @returns [lng,lat]
   */
  gcj02tobd09: typeof gcj02tobd09;
  /**
   * WGS84转GCj02
   * @param lng
   * @param lat
   * @returns [lng,lat]
   */
  wgs84togcj02: typeof wgs84togcj02;
  /**
   * GCJ02 转换为 WGS84
   * @param lng
   * @param lat
   * @returns [lng,lat]
   */
  gcj02towgs84: typeof gcj02towgs84;
  /**
   * 判断是否在国内，不在国内则不做偏移
   * @param lng
   * @param lat
   * @returns {boolean}
   */
  outOfChina: typeof out_of_china;
}
export default UseCoordinate;
