import Geometry from 'ol/geom/Geometry';
import Feature from 'ol/Feature';
import WKTTOOLS from 'terraformer-wkt-parser';
/**
 * 区域处理工具
 */
declare type UseFormatArea = {
  /**
   * 通过wkt字符串或geojson获取中心点
   * @param str wkt/geojson
   */
  getCenterByWktOrGeojson: (str: string) => number[] | null;
  /**
   * 通过geometry获取wkt
   * @param geometry
   */
  writeGeomeGeoms: (geometry: Geometry) => string | null;
  /**
   * 通过wkt/geojson获取geometry
   * @param str wkt/geojson 字符串
   */
  readGeometry: (str?: string | undefined) => Geometry | null;
  /**
   * 合并区域
   * @param list 对象
   * @param param 取list里的参数
   */
  mergeArea: (list: any[], param?: string) => string | null;
  /**
   * 通过feature获取中心点
   * @param feature
   */
  getFeatureCenter: (feature: Feature<any>) => number[] | null;
  /**
   * wkt字符串转geojosn对象
   */
  WKT2GEOJSON: typeof WKTTOOLS.parse;
  /**
   * geojson对象转wkt字符串
   */
  GEOJSON2WKT: typeof WKTTOOLS.convert;
}
export default UseFormatArea;
