/* eslint-disable @typescript-eslint/no-invalid-this */
/* eslint-disable no-param-reassign */
/*
 * @Author: zhenxing xiezhenxing@codvision.com
 * @Date: 2022-05-30 14:20:21
 * @LastEditors: zhenxing xiezhenxing@codvision.com
 * @LastEditTime: 2022-05-30 14:27:25
 * @FilePath: \组件库\src\pages\MapComponent\OlTest\components\OlMyMap\mapTools\UseFormatArea\index.ts
 * @Description: 这是默认设置,请设置`customMade`, 打开koroFileHeader查看配置 进行设置: https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 */
import readGeometry from '../readGeometry';
import { getCenter } from 'ol/extent';
import MultiPolygon from 'ol/geom/MultiPolygon';
import Polygon from 'ol/geom/Polygon';
import OLWKT from 'ol/format/WKT';
import WKTTOOLS from 'terraformer-wkt-parser';
import { Geometry } from 'ol/geom';
import Feature from 'ol/Feature';

/**
 * 区域处理工具
 */
const UseFormatArea = {
  /**
     * 通过wkt字符串或geojson获取中心点
     * @param str wkt/geojson
     */
  getCenterByWktOrGeojson: (str: string) => {
    const f = readGeometry(str);
    if (!f)
      return null;
    return getCenter(f.getExtent());
  },
  readGeometry,
  /**
     * 通过geometry获取wkt
     * @param geometry
   */
  writeGeomeGeoms: (geometry: Geometry) => {
    const wkt = new OLWKT();
    if (!geometry) {
      return null;
    }
    return wkt.writeGeometry(geometry);
  },
  /**
   * 合并区域
   * @param list 对象
   * @param param 取list里的参数
   */
  mergeArea: (list: any[], param?: string) => {
    if (param === void 0) { param = 'wkt'; }
    const geoms = new MultiPolygon([]);
    list.forEach(function (item) {
      try {
        const geometry = readGeometry(item[param!] || '');
        if (geometry instanceof Polygon) {
          geoms.appendPolygon(geometry);
        }
        else if (geometry instanceof MultiPolygon) {
          geometry.getPolygons().forEach(function (subItem) {
            geoms.appendPolygon(subItem);
          });
        }
      }
      catch (error) {
        //
      }
    });
    const geomString = UseFormatArea?.writeGeomeGeoms(geoms);
    if (geomString === 'MULTIPOLYGON EMPTY')
      return null;
    return geomString;
  },
  /**
 * 通过feature获取中心点
 * @param feature
 */
  getFeatureCenter: (feature: Feature<any>) => {
    let _a;
    const newExtent = (_a = feature.getGeometry()) === null || _a === void 0 ? void 0 : _a.getExtent();
    if (!newExtent) {
      return null;
    }
    const position = getCenter(newExtent);
    return position;
  },
  /**
    * wkt字符串转geojosn对象
    */
  WKT2GEOJSON: WKTTOOLS.parse,
  /**
   * geojson对象转wkt字符串
   */
  GEOJSON2WKT: WKTTOOLS.convert,
}


export default UseFormatArea;